/*
 *  LRT.cpp
 *  
 *
 *  Created by Feng Zhou on 6/29/11.
 *  Copyright 2011 University of Kentucky. All rights reserved.
 *
 */
#include <fstream.h>
#include <iostream.h>
#include <cstdlib>
#include <cstring>
#include <math.h>
#include <algorithm>
#include <time.h>
#include "Matrix.h"
#include <Eigen>
#include "mtrand.h"
#include "MLE.h"
#include "LRT.h"

using namespace std;
using namespace Eigen;


double log_uni(int J, int **XIJ, Vector2d mu){
	double L;
	double p;
	for (int j = 0; j < J; j++) {
		p = prob_uni(j, mu);
		if (p > 0)L += XIJ[0][j] * log(p) + XIJ[1][j] * (log(1 - p));
	}
	
	return L;
}

double log_bi(int J, int K, int ***Xnull, Vector3d mu){
	double L;
	double p;
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			p = prob_bi(j, k, mu);
			if (p > 0) L += Xnull[0][j][k] * log(p) + Xnull[1][j][k] * (log(1 - p));
		}
		
	}
	
	return L;
}

double log_indept(int J, int K, int ***Xnull, double ***p){
	double L;
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			if (p[0][j][k]>0 and p[1][j][k] > 0) {
				L += Xnull[0][j][k] * log(p[0][j][k]) +Xnull[1][j][k] *log(p[1][j][k]);
			}
			
		}
	}
	
	return L;
	
}

double LRT(double log_null, double log_alt){
	double ratio;
	ratio = 2 * (log_alt - log_null);
	return ratio;
	
}

double chisq_stat(int I, int J, int K, int ***Xnull, double ***p){
	double chisq, ***expval;
	
	expval = new double **[I];
	
	for (int i = 0; i < I; i++) {
		(expval)[i] = new double *[J];
		for (int j = 0; j < J; j++) {
			(expval)[i][j] = new double [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				expval[i][j][k] = p[i][j][k] * (Xnull[0][j][k] + Xnull[1][j][k]);
				if (expval[i][j][k]>0) {
					chisq += (Xnull[i][j][k] - expval[i][j][k]) * (Xnull[i][j][k] - expval[i][j][k]) / expval[i][j][k];
				}
			}
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			delete [] expval[i][j];
		}
		delete [] expval[i];
	}
	delete [] expval;
	
	return chisq;
}
