/*
 *  bivariate.cpp
 *  
 *
 *  Created by Feng Zhou on 7/28/11.
 *  Copyright 2011 University of Kentucky. All rights reserved.
 *
 */

#include <fstream.h>
#include <iostream.h>
#include <cstdlib>
#include <cstring>
#include <math.h>
#include <algorithm>
#include <time.h>
#include "Matrix.h"
#include <Eigen>
#include "mtrand.h"
#include "suffstat.h"
#include "MLE.h"
#include "LRT.h"
#include "move.h"
#include "length.h"
#include "onetable.h"
#include "read.h"

using namespace std;
using namespace Eigen;



int main(int argc, char *argv[]){
	clock_t t1, t2;
	t1 = clock();
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	int I, J, K, itr2, Skip, BurnIn, ***Xnull, freq = 0;
	
	double ***p, chisq0, chisq;
	
	char outfile1[200], outfile2[200];
	char invocation[200];
	strcpy(invocation, "invocation: ");
	for(int f = 0; f < argc; f++){
		strcat(invocation, argv[f]);
		strcat(invocation, " ");
	}
	cout << invocation << endl; 
	
	read(argc, argv, I, J, K, itr2, Skip, BurnIn, &Xnull);
	
	//	cout << itr <<" " << itr2 << " "<< Skip <<  " "<< BurnIn << endl;
	
	strcpy(outfile1, argv[1]);
	strcat(outfile1, ".table");
	strcpy(outfile2, argv[1]);
	strcat(outfile2, ".LRT");
	
	
	ofstream outTable, outLRT;
	outTable.open(outfile1);
	outLRT.open(outfile2);
	
	p = new double **[I];
	
	for (int i = 0; i < I; i++) {
		(p)[i] = new double *[J];
		for (int j = 0; j < J; j++) {
			(p)[i][j] = new double [K];
		}
	}
	
	
	
	no3way_Indept_IPF(I, J, K, Xnull, &p);
	
	
	chisq0 = chisq_stat(I, J, K, Xnull, p);
	cout << "The Pearson's Chi-Square statistics of the observed table is:" << chisq0 << endl << endl;
	
	srand(time(0));
	irand.seed(time(0));
	int re_run=0;
	
	for(int numOfItr = 0; numOfItr < BurnIn; numOfItr++){
		re_run = 0;
		while(OneTable_indept(I, J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 100) break;
		}
	}
	
	for(int numOfItr = 0; numOfItr < itr2*Skip; numOfItr++){
		re_run = 0;
		while(OneTable_indept(I, J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 100) break;
		}
		no3way_Indept_IPF(I, J, K, Xnull, &p);
		
		chisq = chisq_stat(I, J, K, Xnull, p);
		
		if((numOfItr % Skip) == 0){
			for(int i = 0; i < I; i++){
				for(int j = 0; j < J; j++){
					for (int k = 0; k < K; k++)
						outTable << Xnull[i][j][k] << " ";
				    outTable << endl;
			    }outTable << endl;
		    }outTable << endl;
		}
		
		if((numOfItr % Skip) == 0){
			outLRT << chisq<< endl;
			
			if(chisq0 <= chisq) freq++;
		}outLRT ; 	 
	}
	
	cout << "p-value is: " << ((double) freq)/ ((double)itr2) << endl << endl;
	t2 = clock();
	cout << "Running time is " << (float(t2) - float(t1))/CLOCKS_PER_SEC;
	return 0;
	
}



