#include <fstream.h>
#include <iostream.h>
#include <cstdlib>
#include <cstring>
#include <math.h>
#include <algorithm>
#include <time.h>
#include "Matrix.h"
#include <Eigen>
#include "mtrand.h"

using namespace std;
using namespace Eigen;


unsigned long MTRand_int32::state[n] = {0x0UL};
int MTRand_int32::p = 0;
bool MTRand_int32::init = false;

void MTRand_int32::gen_state() { // generate new state vector
	for (int i = 0; i < (n - m); ++i)
		state[i] = state[i + m] ^ twiddle(state[i], state[i + 1]);
	for (int i = n - m; i < (n - 1); ++i)
		state[i] = state[i + m - n] ^ twiddle(state[i], state[i + 1]);
	state[n - 1] = state[m - 1] ^ twiddle(state[n - 1], state[0]);
	p = 0; // reset position
}

void MTRand_int32::seed(unsigned long s) {  // init by 32 bit seed
	state[0] = s & 0xFFFFFFFFUL; // for > 32 bit machines
	for (int i = 1; i < n; ++i) {
		state[i] = 1812433253UL * (state[i - 1] ^ (state[i - 1] >> 30)) + i;
		// see Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier
		// in the previous versions, MSBs of the seed affect only MSBs of the array state
		// 2002/01/09 modified by Makoto Matsumoto
		state[i] &= 0xFFFFFFFFUL; // for > 32 bit machines
	}
	p = n; // force gen_state() to be called for next random number
}

void MTRand_int32::seed(const unsigned long* array, int size) { // init by array
	seed(19650218UL);
	int i = 1, j = 0;
	for (int k = ((n > size) ? n : size); k; --k) {
		state[i] = (state[i] ^ ((state[i - 1] ^ (state[i - 1] >> 30)) * 1664525UL))
		+ array[j] + j; // non linear
		state[i] &= 0xFFFFFFFFUL; // for > 32 bit machines
		++j; j %= size;
		if ((++i) == n) { state[0] = state[n - 1]; i = 1; }
	}
	for (int k = n - 1; k; --k) {
		state[i] = (state[i] ^ ((state[i - 1] ^ (state[i - 1] >> 30)) * 1566083941UL)) - i;
		state[i] &= 0xFFFFFFFFUL; // for > 32 bit machines
		if ((++i) == n) { state[0] = state[n - 1]; i = 1; }
	}
	state[0] = 0x80000000UL; // MSB is 1; assuring non-zero initial array
	p = n; // force gen_state() to be called for next random number
}



Matrix2 *suff_stat(int J, int K, int ***Xnull){
	Matrix2 *ss = new Matrix2(1,3);
	int X2[J], X3[K];
	
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			(*ss).rowVectors[0].elements[0] = (*ss).rowVectors[0].elements[0] + Xnull[0][j][k];
		}
	}

	
	for (int j = 0; j < J; j++) {
		X2[j] = 0;
		for (int k = 0; k < K; k++) {
			X2[j]=X2[j]+Xnull[0][j][k];
		}
		(*ss).rowVectors[0].elements[1] = (*ss).rowVectors[0].elements[1] + (j+1)*X2[j];
	}
	
	for (int k = 0; k < K; k++) {
		X3[k] = 0;
		for (int j = 0; j < J; j++) {
			X3[k]=X3[k]+Xnull[0][j][k];
		}
		(*ss).rowVectors[0].elements[2] = (*ss).rowVectors[0].elements[2] + (k+1)*X3[k];
	}
	
	return ss;	
	
}

/*

	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			delete [] Xnull[i][j];
		}
		delete [] Xnull[i];
	}
	delete [] Xnull;
		
    return 0;
}*/

double prob_uni(int j, Vector2d mu){
	double p;
	p = exp(mu[0] + mu[1]*(j+1))/(1 + exp(mu[0] + mu[1]*(j+1)));
	return p;
}

double prob_bi(int j, int k, Vector3d mu){
	double p;
	p = exp(mu[0] + mu[1]*(j+1) + mu[2]*(k+1))/(1 + exp(mu[0] + mu[1]*(j+1) + mu[2]*(k+1)));
	return p;
}

void no3way_Indept_IPF(int I, int J, int K, int ***Xnull, double ****p){
	int total = 0, cont = 1;
	double diff = 0, diff_abs = 0, diff_sum = 0;
	int Isum[J][K], Jsum[I][K], Ksum[I][J];
	double tmp[I][J][K], Xold[I][J][K];
	double oldIsum[J][K], oldJsum[I][K], oldKsum[I][J],
	       tmpIsum[J][K], tmpJsum[I][K], tmpKsum[I][J];
	
	for(int i = 0; i < I; i++){
		for(int j = 0; j < J; j++){
			for (int k = 0; k < K; k++) {
				total += Xnull[i][j][k];
			}
		}
	}

	
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			Isum[j][k] = 0;
			for (int i = 0; i < I; i++) {
				Isum[j][k] += Xnull[i][j][k];
			
			}
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int k = 0; k < K; k++) {
			Jsum[i][k] = 0;
			for (int j = 0; j < J; j++) {
				Jsum[i][k] += Xnull[i][j][k];
			}
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			Ksum[i][j] = 0;
			for (int k = 0; k < K; k++) {
				Ksum[i][j] += Xnull[i][j][k];
			}
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				Xold[i][j][k] = 0.001;
			}
		}
	}
	
	while (cont = 1) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				oldIsum[j][k] = 0.0;
				for (int i = 0; i < I; i++) {
					oldIsum[j][k] += Xold[i][j][k];
	            
				}
			}
		}
		
		
		for(int j = 0; j < J; j++){
			for(int k = 0; k < K; k++){
				if(oldIsum[j][k] > 0){
					for (int i = 0; i < I; i++){ 
						tmp[i][j][k] = Xold[i][j][k] * ((double)Isum[j][k]) / oldIsum[j][k]/total;	
					}		
				}
			}
		}
		
		for (int i = 0; i < I; i++){
			for (int j = 0; j < J; j++){
				for (int k = 0; k < K; k++) {
					diff += (tmp[i][j][k] - Xold[i][j][k]);
					if (diff < 0) {diff_abs = -diff;}
					else {diff_abs = diff;}
					diff_sum +=diff_abs;	
		
				}
			}
		}
		
		
		if (diff_sum < 0.000001) {
			for (int j = 0; j < J; j++) {
				for (int k = 0; k < K; k++) {
					tmpIsum[j][k] = 0.0;
					for (int i = 0; i < I; i++) {
						tmpIsum[j][k] += tmp[i][j][k];
					}
				}
			}
			
			
			for(int j = 0; j < J; j++){
				for (int k = 0; k < K; k++) {
//					if (tmpIsum[j][k]>0) {
						for(int i = 0; i < I; i++){
							(*p)[i][j][k] = tmp[i][j][k]/tmpIsum[j][k];
						}
//					}
//					else {
//						for(int i = 0; i < I; i++){
//							(*p)[i][j][k] = tmp[i][j][k]/0.0001;
//						}
//					}


				}
			}
			
			
		
		    
			cont = 0;
			break;	
			
		}
		
	
		
		else {
			for(int j = 0; j < J; j++){
				for (int k = 0; k < K; k++) {
					for(int i = 0; i < I; i++){
						Xold[i][j][k] = tmp[i][j][k];
					}
				}
			}
		}
		

		
		for (int i = 0; i < I; i++) {
			for (int k = 0; k < K; k++) {
				oldJsum[i][k] = 0.0;
				for (int j = 0; j < J; j++) {
					oldJsum[i][k] += Xold[i][j][k];
				}
			}
		}
		
		for (int i = 0; i < I; i++) {
			for (int k = 0; k < K; k++) {
				if(oldJsum[i][k] > 0){
				for (int j = 0; j < J; j++) {
						tmp[i][j][k] = Xold[i][j][k] * ((double)Jsum[i][k]) / oldJsum[i][k]/total;	
					}	
				}
			}
		}
		
		diff = 0;
		diff_abs = 0;
		diff_sum = 0;
		
		for(int i = 0; i < I; i++){
			for(int j = 0; j < J; j++){
				for (int k = 0; k < K; k++) {
					diff += (tmp[i][j][k] - Xold[i][j][k]);
					if (diff < 0) {diff_abs = -diff;}
					else {diff_abs = diff;}
					diff_sum +=diff_abs;	
					
				}
			}
		}

		
		if (diff_sum < 0.000001) {
			for (int j = 0; j < J; j++) {
				for (int k = 0; k < K; k++) {
					tmpIsum[j][k] = 0.0;
					for (int i = 0; i < I; i++) {
						tmpIsum[j][k] += tmp[i][j][k];
					}
				}
			}
			
			for(int j = 0; j < J; j++){
				for (int k = 0; k < K; k++) {
//					cout << "tmpIsum"<<tmpIsum[j][k]<<endl;
//					if (tmpIsum[j][k]>0) {
						for(int i = 0; i < I; i++){
							(*p)[i][j][k] = tmp[i][j][k]/tmpIsum[j][k];
						}
//					}
//					else {
//						for(int i = 0; i < I; i++){
//							(*p)[i][j][k] = tmp[i][j][k]/0.0001;
//						}
//					}
				}
			}
			cont = 0;
		
			break;
			
		}
		
		
		else {
			for(int i = 0; i < I; i++){
				for (int k = 0; k < K; k++) {
					for(int j = 0; j < J; j++){
						Xold[i][j][k] = tmp[i][j][k];
					}
				}
			}
		}

		
		
		
		for (int i = 0; i < I; i++) {
			for (int j = 0; j < J; j++) {
				oldKsum[i][j] = 0.0;
				for (int k = 0; k < K; k++) {
					oldKsum[i][j] += Xold[i][j][k];
				}
			}
		}
		
		for (int i = 0; i < I; i++) {
			for (int j = 0; j < J; j++) {
				if(oldKsum[i][j] > 0){
				for (int k = 0; k < K; k++) {
			        tmp[i][j][k] = Xold[i][j][k] * ((double)Ksum[i][j]) / oldKsum[i][j]/total;	
					}
				}
			}
		}
		
		diff = 0;
		diff_abs = 0;
		diff_sum = 0;
		
		for(int i = 0; i < I; i++){
			for(int j = 0; j < J; j++){
				for (int k = 0; k < K; k++) {
					diff += (tmp[i][j][k] - Xold[i][j][k]);
					if (diff < 0) {diff_abs = -diff;}
					else {diff_abs = diff;}
					diff_sum +=diff_abs;	
					
				}
			}
		}
		
		if (diff_sum < 0.000001) {
			for (int j = 0; j < J; j++) {
				for (int k = 0; k < K; k++) {
					tmpIsum[j][k] = 0.0;
					for (int i = 0; i < I; i++) {
						tmpIsum[j][k] += tmp[i][j][k];
					}
				}
			}
			
			for(int j = 0; j < J; j++){
				for (int k = 0; k < K; k++) {
//					cout << "tmpIsum"<<tmpIsum[j][k]<<endl;
//					if (tmpIsum[j][k]>0) {
						for(int i = 0; i < I; i++){
							(*p)[i][j][k] = tmp[i][j][k]/tmpIsum[j][k];
						}
//					}
//					else {
//						for(int i = 0; i < I; i++){
//							(*p)[i][j][k] = tmp[i][j][k]/0.0001;
//						}
//					}
				}
			}
			cont = 0;
		
			break;
		}
		
		else {
			for(int i = 0; i < I; i++){
				for (int j = 0; j < J; j++) {
					for(int k = 0; k < K; k++){
						Xold[i][j][k] = tmp[i][j][k];
					}
				}
			}
		}	
		
	
		
	}
	
//	for (int i = 0; i < I; i++) {
//		for (int j = 0; j < J; j++) {
//			for (int k = 0; k < K; k++) {
//				cout << (*p)[i][j][k] << " "; 
//			}
//			cout << endl;
//		}
//		cout << endl;
//	}
//	cout << endl; 
	
}



Vector2d para_1st_uni(int J, int *XJ, Vector2d mu, Matrix2 *ss){
	double u[2] = {0};
	Vector2d uPtr;
	for (int j = 0; j < J; j++) {
		u[0] = u[0] - XJ[j]*prob_uni(j, mu); 
		u[1] = u[1] - (j+1)*XJ[j]*prob_uni(j, mu);
		
	}
	u[0] = u[0] + (*ss).rowVectors[0].elements[0];
	u[1] = u[1] + (*ss).rowVectors[0].elements[1];
	
	uPtr << u[0], u[1];	
	return uPtr;
}

Vector3d para_1st_bi(int J, int K, int **XJK, Vector3d mu, Matrix2 *ss){
	double u[3] = {0};
	Vector3d uPtr;
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			u[0] = u[0] - XJK[j][k] * prob_bi(j, k, mu); 
			u[1] = u[1] - (j+1)*XJK[j][k] * prob_bi(j, k, mu);
			u[2] = u[2] - (k+1)*XJK[j][k] * prob_bi(j, k, mu);
		}
	}
	u[0] = u[0] + (*ss).rowVectors[0].elements[0];
	u[1] = u[1] + (*ss).rowVectors[0].elements[1];
	u[2] = u[2] + (*ss).rowVectors[0].elements[2];
	
	uPtr << u[0], u[1], u[2];	
	return uPtr;
}

Matrix2 *hesse_uni(int J, int *XJ, Vector2d mu){
	Matrix2 *h = new Matrix2(2,2);
	double p;
	for (int j = 0; j < J; j++) {
		p = prob_uni(j, mu);
		(*h).rowVectors[0].elements[0] = (*h).rowVectors[0].elements[0] - XJ[j]*p*(1-p);
		(*h).rowVectors[0].elements[1] = (*h).rowVectors[0].elements[1] - XJ[j]*p*(1-p)*(j+1);
		(*h).rowVectors[1].elements[0] = (*h).rowVectors[1].elements[0] - XJ[j]*p*(1-p)*(j+1);
		(*h).rowVectors[1].elements[1] = (*h).rowVectors[1].elements[1] - XJ[j]*p*(1-p)*(j+1)*(j+1);
	}
	
	return h;
	
}
/*
int main(){
	int J =5;
	int *XJ;
	XJ =new int[J];
	int M[5] ={10,20,1,13,5};
	for (int i = 0; i < 5; i++) {
		XJ[i] = M[i];
	}
	Vector2d mu;
	mu << 0, 0;
	Matrix2 *h = new Matrix2(2,2);
	Matrix2d hinv, hesse;
	h = hesse_uni(J, XJ, mu);
	cout << (*h).rowVectors[0].elements[0] << " "<<(*h).rowVectors[0].elements[1] << " "<<
	(*h).rowVectors[1].elements[0] << " "<<(*h).rowVectors[1].elements[1] << " ";
	hesse << (*h).rowVectors[0].elements[0], (*h).rowVectors[0].elements[1],
	(*h).rowVectors[1].elements[0],(*h).rowVectors[1].elements[1];	
	hinv = hesse.inverse();	
	cout << hinv;
	
	
}*/

Matrix2 *hesse_bi(int J, int K, int **XJK, Vector3d mu){
	Matrix2 *h = new Matrix2(3,3);
	double p;
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			p = prob_bi(j, k, mu);
			(*h).rowVectors[0].elements[0] = (*h).rowVectors[0].elements[0] - XJK[j][k]*p*(1-p);
			(*h).rowVectors[0].elements[1] = (*h).rowVectors[0].elements[1] - XJK[j][k]*p*(1-p)*(j+1);
			(*h).rowVectors[0].elements[2] = (*h).rowVectors[0].elements[2] - XJK[j][k]*p*(1-p)*(k+1);
			(*h).rowVectors[1].elements[1] = (*h).rowVectors[1].elements[1] - XJK[j][k]*p*(1-p)*(j+1)*(j+1);
			(*h).rowVectors[1].elements[2] = (*h).rowVectors[1].elements[2] - XJK[j][k]*p*(1-p)*(j+1)*(k+1);
			(*h).rowVectors[2].elements[2] = (*h).rowVectors[2].elements[2] - XJK[j][k]*p*(1-p)*(k+1)*(k+1);
		}
	}
	(*h).rowVectors[1].elements[0] = (*h).rowVectors[0].elements[1];
	(*h).rowVectors[2].elements[0] = (*h).rowVectors[0].elements[2];
	(*h).rowVectors[2].elements[1] = (*h).rowVectors[1].elements[2];
	return h;
	
}

Vector2d newton_uni(int J, int *XJ, Matrix2 *ss){
	Matrix2 *h;
	Matrix2d hesse, hinv;
	Vector2d u, mu, mu_new, diff, diff_abs;
	mu << 0, 0;
	int condition = 1, cont = 0;
	for (int i = 0; condition = 1; i++) {
		h = hesse_uni(J, XJ, mu);	
		hesse << (*h).rowVectors[0].elements[0], (*h).rowVectors[0].elements[1],
		(*h).rowVectors[1].elements[0],(*h).rowVectors[1].elements[1];	
		hinv = hesse.inverse();		
//		cout << "hinv_uni"<<hinv<<endl ;
		u = para_1st_uni(J, XJ, mu, ss);
//		cout << "u_uni"<<u<< endl ;
		mu_new = mu - hinv * u;
	    diff = mu_new-mu;
		for (int i = 0; i < 2; i++) {
			if (diff[i] < 0) diff_abs[i] = -diff[i];
			else diff_abs[i] = diff[i];

		}
		
		if (diff_abs.sum()< 0.0000001){
			condition = 0;
			mu = mu_new;
			break;
		}
		else {
			mu = mu_new;
		}
		cont = cont + 1;
		if (cont > 100) {
			mu << -10000, -10000;
			break;
		}
	}	
	cout << "cont="<<cont<<endl;
	return mu;
	
}
/*
int main(){
	int J =5;
	int *XJ;
	XJ =new int[J];
	int M[5] ={10,20,1,13,5};
	for (int i = 0; i < 5; i++) {
		XJ[i] = M[i];
	}
	Matrix2 *ss = new Matrix2(1,2);
	(*ss).rowVectors[0].elements[0] = 34;
	(*ss).rowVectors[0].elements[1] = 93;
	Vector2d mu1;
	mu1 = newton_uni(J, XJ, ss);
	cout << mu1 << endl;
	
	
}*/



Vector3d newton_bi(int J, int K, int **XJK, Matrix2 *ss){
	Matrix2 *h;
	Matrix3d hesse, hinv;
	Vector3d u, mu, mu_new, diff, diff_abs;
	mu << 0, 0, 0;
	int condition = 1, cont = 0;
	for (int i = 0; condition = 1; i++) {
		h = hesse_bi(J, K, XJK, mu);	
		hesse << (*h).rowVectors[0].elements[0], (*h).rowVectors[0].elements[1], (*h).rowVectors[0].elements[2],
				 (*h).rowVectors[1].elements[0], (*h).rowVectors[1].elements[1], (*h).rowVectors[1].elements[2],
		         (*h).rowVectors[2].elements[0], (*h).rowVectors[2].elements[1], (*h).rowVectors[2].elements[2];	
		hinv = hesse.inverse();	
//		cout << "hinv_bi"<<hinv<<endl;
		u = para_1st_bi(J, K, XJK, mu, ss);
//		cout << "u_bi"<<u<<endl;
		mu_new = mu - hinv * u;
	    diff = mu_new-mu;
		for (int i = 0; i < 3; i++) {
			if (diff[i] < 0) diff_abs[i] = -diff[i];
			else diff_abs[i] = diff[i];
			
		}
		
		if (diff_abs.sum()< 0.0000001){
			condition = 0;
			mu = mu_new;
			break;
		}
		else {
			mu = mu_new;
		}
		cont = cont + 1;
		if (cont > 100) {
			mu << -10000, -10000, -10000;
			break;
		}
		
	}	
//	cout << "cont="<<cont<<endl;
	return mu;
	
}
/*
int main(){
	int J = 5, K = 2;
	int *XJ;
	XJ =new int[J];
	int M[5] ={10,20,1,13,5};
	for (int i = 0; i < 5; i++) {
		XJ[i] = M[i];
	}
	
	int **XJK;
	
	XJK = new int *[J];
	for (int j = 0; j < J; j++) {
		XJK[j] = new int[K]; 
	}
	
	int N[5][2]={{1,9},{0,20},{0,1},{4,9},{0,5}};
	
	for (int j = 0; j < J; j++) 
		for (int k = 0; k < K; k++) 
			XJK[j][k] = N[j][k];
			

	
	
	Matrix2 *ss = new Matrix2(1,3);
	(*ss).rowVectors[0].elements[0] = 34;
	(*ss).rowVectors[0].elements[1] = 93;
	(*ss).rowVectors[0].elements[2] = 68;
	Vector3d mu1;
	mu1 = newton_bi(J, K, XJK, ss);
	cout << mu1 << endl;
	
	
}*/




double log_uni(int J, int **XIJ, Vector2d mu){
	double L;
	double p;
	for (int j = 0; j < J; j++) {
		p = prob_uni(j, mu);
		if (p > 0) {
			L += XIJ[0][j] * log(p) + XIJ[1][j] * (log(1 - p));
		}		
	}
	
	return L;
}

double log_bi(int J, int K, int ***Xnull, Vector3d mu){
	double L;
	double p;
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			p = prob_bi(j, k, mu);
			if (p > 0) {
				L += Xnull[0][j][k] * log(p) + Xnull[1][j][k] * (log(1 - p));
			}
		}
		
	}
	
	return L;
}

double log_indept(int J, int K, int ***Xnull, double ***p){
	double L;
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			if (p[0][j][k]>0 and p[1][j][k] > 0) {
				L += Xnull[0][j][k] * log(p[0][j][k]) +Xnull[1][j][k] *log(p[1][j][k]);
			}
			
		}
	}
	
	return L;
	
}

double chisq_stat(int I, int J, int K, int ***Xnull, double ***p){
	double chisq, ***expval;
	
	expval = new double **[I];
	
	for (int i = 0; i < I; i++) {
		(expval)[i] = new double *[J];
		for (int j = 0; j < J; j++) {
			(expval)[i][j] = new double [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				expval[i][j][k] = p[i][j][k] * (Xnull[0][j][k] + Xnull[1][j][k]);
				if (expval[i][j][k]>0) {
					chisq += (Xnull[i][j][k] - expval[i][j][k]) * (Xnull[i][j][k] - expval[i][j][k]) / expval[i][j][k];
				}
			}
		}
	}
	
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			delete [] expval[i][j];
		}
		delete [] expval[i];
	}
	delete [] expval;
	
	return chisq;
}


/*
double chisq_stat(int I, int J, int K, int ***Xnull, Vector3d mu){
	double chisq, ***expval, p;
	
	expval = new double **[I];
	
	for (int i = 0; i < I; i++) {
		(expval)[i] = new double *[J];
		for (int j = 0; j < J; j++) {
			(expval)[i][j] = new double [K];
		}
	}
	

		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				p = prob_bi(j, k, mu);
				expval[0][j][k] = p * (Xnull[0][j][k] + Xnull[1][j][k]);
				expval[1][j][k] = (1 - p) * (Xnull[0][j][k] + Xnull[1][j][k]);
				
			}
		}
	
	for (int i = 0; i < I; i++)
		for (int j = 0; j < J; j++)
			for (int k = 0; k < K; k++)
	if (expval[0][j][k]>0 and expval[1][j][k]>0) {
		chisq += (Xnull[i][j][k] - expval[i][j][k]) * (Xnull[i][j][k] - expval[i][j][k]) / expval[i][j][k];
	}
	
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			delete [] expval[i][j];
		}
		delete [] expval[i];
	}
	delete [] expval;
	
	return chisq;
}

*/
double LRT(double log_null, double log_alt){
	double ratio;
	ratio = 2 * (log_alt - log_null);
	return ratio;
	
}

/*
int main(int argc, char *argv[]){
	char filename[200], filename11[200];
	int I, J, K;
	
	strcpy(filename, argv[1]);
	sprintf(filename11, "%s.mod",filename);
	ifstream in(filename11);
	if(!in){
		cerr << "Error on reading the input file." << endl;
		exit(0); 
	}
	in >> I >> J >> K;
//	cout << I << " " << J << " " << K << " " << endl;
	
	int ***Xnull;
	
	Xnull = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(Xnull)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(Xnull)[i][j] = new int [K];
		}
	}
	
//	for (int i = 0; i < I; i++) {
//		for (int j = 0; j < J; j++) {
//			for (int k = 0; k < K; k++) {
//				in >> Xnull[i][j][k];
//				cout << Xnull[i][j][k] << " ";
//			}
//			cout << endl;
//		}
//		cout << endl;
//	}
	
	Matrix2 *stats;
	stats = suff_stat(J, K, Xnull);
	
	
//	for (int t = 0; t < 3; t++) {
//		cout << (*stats).rowVectors[0].elements[t] << " ";
//	} 
//	cout << endl;
	
	Matrix2 *ss_null = new Matrix2(1,2);
	
	for (int i = 0; i < 2; i++) {
		(*ss_null).rowVectors[0].elements[i] = (*stats).rowVectors[0].elements[i];
//		cout << (*ss_null).rowVectors[0].elements[i] << " ";
	}
//	cout << endl;
	
	int **XJK;
	
	XJK = new int *[J];
	for (int j = 0; j < J; j++) {
		XJK[j] = new int[K]; 
	}
	
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			XJK[j][k] = Xnull[0][j][k] + Xnull[1][j][k];
//			cout << XJK[j][k] << " ";
		}
//		cout << endl;
	}
//	cout << endl;
	
	int **XIJ;
	
	XIJ = new int *[I];
	for (int i = 0; i < I; i++) {
		XIJ[i] = new int[J]; 
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			XIJ[i][j] = 0;
			for (int k = 0; k < K; k++) {
				XIJ[i][j] = XIJ[i][j] + Xnull[i][j][k];
				
			}
//			cout << XIJ[i][j] << " ";
		}
//		cout << endl;
	}
//	cout << endl;
	
	int *XJ;
	XJ =new int[J];
	
	for (int j = 0; j < J; j++) {
		XJ[j] = 0;
		for (int i = 0; i < I; i++) {
			for (int k = 0; k < K; k++) {
				XJ[j] = XJ[j] + Xnull[i][j][k];				
			}			
		}
//		cout << XJ[j] << " ";
	}
//	cout << endl;
	
	Vector2d mu1;
	mu1 = newton_uni(J, XJ, ss_null);
	cout << mu1 << endl;
	
	
	Vector3d mu2;
	mu2 = newton_bi(J, K, XJK, stats);
	cout << mu2 << endl;
	
	double L1;
	L1 = log_uni(J, XIJ, mu1);
	cout << L1 << " ";
	
	double L2;
	L2 = log_bi(J, K, Xnull, mu2);
	cout << L2 << endl;
	
	double ratio1;
	ratio1 = LRT(L1, L2);
	cout << ratio1;
	cout << endl;
	
	double ***p;
	
	p = new double **[I];
	
	for (int i = 0; i < I; i++) {
		(p)[i] = new double *[J];
		for (int j = 0; j < J; j++) {
			(p)[i][j] = new double [K];
		}
	}
	
    no3way_Indept_IPF(I, J, K, Xnull, &p);
	
	double L3;
	L3 = log_indept(J, K, Xnull, p);
	cout << L3;
	cout << endl;
	
	double ratio2;
	ratio2 = LRT(L2, L3);
	cout << "LRT" << ratio2;
	
    return 0;
}

*/
void swap(int  x, int  y){
	int tmp;
	tmp = x;
	x = y;
	y = tmp;
}

void bubblesort(int n, Matrix2 *cell) {
	for (int i = 0; i < n-1; i++)
		for (int j = n-1; j > i; --j)
			if ((*cell).rowVectors[0].elements[j] < (*cell).rowVectors[0].elements[j-1])
				swap((*cell).rowVectors[0].elements[j] , (*cell).rowVectors[0].elements[j-1]);
}

void intsort(int size, int a[]) {
	int hold;
	for (int pass = 1; pass < size; pass++) {
		for (int j = 0; j < size - 1; j++) {
			if (a[j] > a[j + 1]) {
				hold = a[j];
				a[j] = a[j + 1];
				a[j + 1] = hold;
			}
		}
	}
	
}

static void getThree (int *dst, int J, MTRand_int32& irand) {
    int sz, pos, i, src[J];
    for (i = 0; i < sizeof(src)/sizeof(*src); i++)
        src[i] = i;
    sz = J;
    for (i = 0; i < 3; i++) {
        pos = irand() % sz;
        dst[i] = src[pos];
        src[pos] = src[sz-1];
        sz--;
    }
}

Matrix2 *rmove_uni(int J, MTRand_int32& irand){
	int i = 0, temp;
    Matrix2 *cell = new Matrix2(1,4);
	Matrix2 *jcell = new Matrix2(1,4);

    
	int numbers[3];
    getThree (numbers, J, irand);
	
//	cout << numbers[0]<<" "<<numbers[1]<<" "<<numbers[2]<<endl;
	
	intsort(3, numbers);

	for (int i = 0; i < 3; i++) {
		(*cell).rowVectors[0].elements[i] = numbers[i];
//		cout << (*cell).rowVectors[0].elements[i] << " ";
	}
//	cout << endl;
	

	
	
	if ((*cell).rowVectors[0].elements[1] - (*cell).rowVectors[0].elements[0] <= (*cell).rowVectors[0].elements[2] - (*cell).rowVectors[0].elements[1]) {
		(*cell).rowVectors[0].elements[3] = (*cell).rowVectors[0].elements[2] - ((*cell).rowVectors[0].elements[1] - (*cell).rowVectors[0].elements[0]);
//	    cout << (*cell).rowVectors[0].elements[0] << " "<<(*cell).rowVectors[0].elements[1]<< " "<<(*cell).rowVectors[0].elements[2]<< " "<<(*cell).rowVectors[0].elements[3]<<endl;
	}
	
	else {
		(*cell).rowVectors[0].elements[3] = (*cell).rowVectors[0].elements[0] + ((*cell).rowVectors[0].elements[2] - (*cell).rowVectors[0].elements[1]);
//      cout << (*cell).rowVectors[0].elements[0] << " "<<(*cell).rowVectors[0].elements[1]<< " "<<(*cell).rowVectors[0].elements[2]<< " "<<(*cell).rowVectors[0].elements[3]<<endl;
	}

	bubblesort(4, cell);
	
	if (irand() % 2 == 0) {
		(*jcell).rowVectors[0].elements[0] = (*cell).rowVectors[0].elements[0];
		(*jcell).rowVectors[0].elements[1] = (*cell).rowVectors[0].elements[3];
		(*jcell).rowVectors[0].elements[2] = (*cell).rowVectors[0].elements[1];
		(*jcell).rowVectors[0].elements[3] = (*cell).rowVectors[0].elements[2];
	}
	
	else {
		(*jcell).rowVectors[0].elements[0] = (*cell).rowVectors[0].elements[1];
		(*jcell).rowVectors[0].elements[1] = (*cell).rowVectors[0].elements[2];
		(*jcell).rowVectors[0].elements[2] = (*cell).rowVectors[0].elements[0];
		(*jcell).rowVectors[0].elements[3] = (*cell).rowVectors[0].elements[3];
	}
    
//	for (int i = 0; i < 4; i++) {
//		cout << (*jcell).rowVectors[0].elements[i] << " ";
//	}
//	cout << endl; 
 
	return jcell;
	
}

/*
int main(){
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	srand(time(0));
	irand.seed(time(0));
    int t=0;
    int J = 3;
    Matrix2 *cell;
	cell = rmove_uni(J, irand);
			
    cout << endl;
    for (int i = 0; i < 4; i++) {
        cout << (*cell).rowVectors[0].elements[i] << " " ;
    }
 
    return 0;	
 
}*/

static void getTwo (int *dst, int num, MTRand_int32& irand) {
    int sz, pos, i, src[num];
    for (i = 0; i < sizeof(src)/sizeof(*src); i++)
        src[i] = i;
    sz = num;
    for (i = 0; i < 2; i++) {
        pos = irand() % sz;
        dst[i] = src[pos];
        src[pos] = src[sz-1];
        sz--;
    }
}

Matrix2 *rmove_bi(int I, int J, int K, MTRand_int32& irand){
	int jstar, kstar, j1, j2, k1, k2, j[2], k[2];
	jstar = 1 + irand() % J;
//	cout << jstar << endl;
	Matrix2 *jk = new Matrix2(4,2);
	
	if (jstar == J) {kstar = 1 + (irand() % (K - 1));}
	else if (jstar == 1) {kstar = 2 + (irand() % (K - 1));}
	else {kstar = 1 + (irand() % K);}
//	cout << kstar<<endl;
	int numbers[2];
    getTwo (numbers, jstar*kstar, irand);
	
	numbers[0] = numbers[0] + 1;
	numbers[1] = numbers[1] + 1;
	
//	cout << numbers[0]<< " "<<numbers[1]<<endl;
	
	if (numbers[0] % kstar == 0) {
		j1 = (numbers[0] - numbers[0] % kstar) / kstar;
		k1 = kstar;
	}
	else {
		j1 = (numbers[0] - numbers[0] % kstar) / kstar + 1;
		k1 = numbers[0] % kstar;
	}
	
	if (numbers[1] % kstar == 0) {
		j2 = (numbers[1] - numbers[1] % kstar) / kstar;
		k2 = kstar;
	}
	else {
		j2 = (numbers[1] - numbers[1] % kstar) / kstar + 1;
		k2 = numbers[1] % kstar;
	}
	
	j[0] = j1;
	j[1] = j2;
	k[0] = k1;
	k[1] = k2;
	
	j1 = J - jstar;
	k1 = K - kstar;
	
	if (irand() % 2 == 0) {
		(*jk).rowVectors[0].elements[0] = j[0] - 1;
		(*jk).rowVectors[0].elements[1] = k[0] - 1;
		(*jk).rowVectors[1].elements[0] = j[1] - 1;
		(*jk).rowVectors[1].elements[1] = k[1] - 1;
		(*jk).rowVectors[2].elements[0] = j[0] + j1 - 1;
		(*jk).rowVectors[2].elements[1] = k[0] + k1 - 1;
		(*jk).rowVectors[3].elements[0] = j[1] + j1 - 1;
		(*jk).rowVectors[3].elements[1] = k[1] + k1 - 1;
	}
	else {
		(*jk).rowVectors[0].elements[0] = j[0] - 1;
		(*jk).rowVectors[0].elements[1] = K - k[0];
		(*jk).rowVectors[1].elements[0] = j[1] - 1;
		(*jk).rowVectors[1].elements[1] = K - k[1];
		(*jk).rowVectors[2].elements[0] = j[0] + j1 - 1;
		(*jk).rowVectors[2].elements[1] = K - k[0] - k1;
		(*jk).rowVectors[3].elements[0] = j[1] + j1 - 1;
		(*jk).rowVectors[3].elements[1] = K - k[1] - k1;
	}

	
	return jk;
	
	
}

Matrix2 *rmove_indept(int J, int K, MTRand_int32& irand){
	Matrix2 *jk = new Matrix2(2,2);
	
	int numbers1[2], numbers2[2];
    getTwo (numbers1, J, irand);
	
//	cout <<"J is"<< numbers1[0]<<" "<<numbers1[1]<<endl;
	
	(*jk).rowVectors[0].elements[0] = numbers1[0];
	(*jk).rowVectors[0].elements[1] = numbers1[1];
	

    getTwo (numbers2, K, irand);
	(*jk).rowVectors[1].elements[0] = numbers2[0];
	(*jk).rowVectors[1].elements[1] = numbers2[1];
	
//	cout << "K is"<<numbers2[0]<<" "<<numbers2[1]<<endl;
	
//	for (int i = 0; i < 2; i++) {
//		for (int j = 0; j < 2; j++) {
//			cout << (*jk).rowVectors[i].elements[j] << " " ;
//		}
		
//	}
	return jk;
	
}


/*

int main(){
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	int I = 2, J = 2, K = 5;
	Matrix2 *cell;
	srand(time(0));
	irand.seed(time(0));
	cell = rmove_bi(I,J,K, irand);
	cout << endl;
	for (int i = 0; i < 4; i++) {
		for (int j = 0; j < 2; j++) {
			cout << (*cell).rowVectors[i].elements[j] << " " ;
		}
		
	}
	
	return 0;	
	
}

*/

int maxl_uni(int I, int J, Matrix2 *cell, int **XIJ){
	int l = 1, cont =1, temp, **tmp;
	
	tmp = new int *[I];
	for (int i = 0; i < I; i++) {
		tmp[i] = new int[J]; 
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			tmp[i][j] = XIJ[i][j];
		}
	}
	
	while (cont = 1) {
		int flag1 = 0, flag2 = 0;
		
		for (int i = 0; i < 2; i++) {
			temp = (*cell).rowVectors[0].elements[i];
			tmp[0][temp] = tmp[0][temp] - 1;
			if (tmp[0][temp] == 0) {flag1++;}
			if (tmp[0][temp] < 0) {flag2++;}
		}
		
		for (int i = 2; i < 4; i++) {
			temp = (*cell).rowVectors[0].elements[i];
			tmp[1][temp] = tmp[1][temp] - 1;
			if (tmp[0][temp] == 0) {flag1++;}
			if (tmp[0][temp] < 0) {flag2++;}
		}
	
		if (flag2 > 0) {
			l = l - 1;
			break;
		}
		
		if (flag1 > 0) {
			break;
		}
		
		if (flag1 == 0 & flag2 == 0) {l = l + 1;}
		
	}
	
	for(int i = 0; i < I; i++) delete tmp[i];
	delete tmp;
	
	return l;
}

/*
 
 int main(){
 int **XIJ, I = 2, J = 8;
 int M[2][8] = {{3,17,12,16,12,8,16,8}, {153,235,272,255,127,77,83,35}};
 
 Matrix2 *cell = new Matrix2(1,4);
 (*cell).rowVectors[0].elements[0] = 1;
 (*cell).rowVectors[0].elements[1] = 5;
 (*cell).rowVectors[0].elements[2] = 2;
 (*cell).rowVectors[0].elements[3] = 4;
 
 
 XIJ = new int *[I];
 for (int i = 0; i < I; i++) {
 XIJ[i] = new int[J]; 
 }
 
 for (int i = 0; i < I; i++) {
 for (int j = 0; j < J; j++) {
 XIJ[i][j] = M[i][j];
 
 }
 
 }
 
 int l;
 l = maxl_uni(I, J, cell, XIJ);
 cout << l;
 
 return 0;
 
 }*/

int maxl_bi(int I, int J, int K, Matrix2 *cell, int ***Xnull){
	int l = 1, cont =1, tempj, tempk;
	int ***tmp;
	
    tmp = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(tmp)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(tmp)[i][j] = new int [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				tmp[i][j][k] = (Xnull)[i][j][k];
			}
		}
	}
	while (cont = 1) {
		int flag1 = 0, flag2 = 0;
		
		for (int i = 1; i < 3; i++) {
			tempj = (*cell).rowVectors[i].elements[0];
			tempk = (*cell).rowVectors[i].elements[1];
			(tmp)[0][tempj][tempk] = (tmp)[0][tempj][tempk] - 1;
			if ((tmp)[0][tempj][tempk] == 0) {flag1++;}
			if ((tmp)[0][tempj][tempk] < 0) {flag2++;}
		}
	
		
		for (int i = 0; i < 4; i+=3) {
			tempj = (*cell).rowVectors[i].elements[0];
			tempk = (*cell).rowVectors[i].elements[1];
			(tmp)[1][tempj][tempk] = (tmp)[1][tempj][tempk] - 1;
			if ((tmp)[1][tempj][tempk] == 0) {flag1++;}
			if ((tmp)[1][tempj][tempk] < 0) {flag2++;}

			
		}
		if (flag2 > 0) {
			l = l - 1;
			break;
		}
		
		if (flag1 > 0) {
			break;
		}
		
		if (flag1 == 0 & flag2 == 0) {l = l + 1;}
		
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			delete [] tmp[i][j];
		}
		delete [] tmp[i];
	}
	delete [] tmp;
	
	return l;
		

	
}



/*

int main(int argc, char *argv[]){
	char filename[200], filename11[200];
	int I, J, K;
	
	strcpy(filename, argv[1]);
	sprintf(filename11, "%s.mod",filename);
	ifstream in(filename11);
	if(!in){
		cerr << "Error on reading the input file." << endl;
		exit(0); 
	}
	in >> I >> J >> K;
	
	int ***Xnull;
	
	Xnull = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(Xnull)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(Xnull)[i][j] = new int [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				in >> Xnull[i][j][k];
			}
		}
	}
	

	
	Matrix2 *cell = new Matrix2(4,2);
	(*cell).rowVectors[0].elements[0] = 0;
	(*cell).rowVectors[1].elements[0] = 4;
	(*cell).rowVectors[2].elements[0] = 3;
	(*cell).rowVectors[3].elements[0] = 7;
	(*cell).rowVectors[0].elements[1] = 6;
	(*cell).rowVectors[1].elements[1] = 5;
	(*cell).rowVectors[2].elements[1] = 1;
	(*cell).rowVectors[3].elements[1] = 0;
	
	
	
 
	int l;
	l = maxl_bi(cell, Xnull);
	cout << l;
	
	return 0;
	
}

*/

int dist(int I, int J, int icell, int jcell, int K, int ***Xnull, MTRand_int32& irand){
    
	int flag = 0, cont = 1, kstar, k;	
	int ***tmp;
	
    tmp = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(tmp)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(tmp)[i][j] = new int [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				tmp[i][j][k] = (Xnull)[i][j][k];
			}
		}
	}
    

	while(cont = 1){
	kstar = irand() % K;
		cout << kstar;
//	cout << kstar << " ";
		
	if (icell == 0 or icell == 1) {
		tmp[0][jcell][kstar] = tmp[0][jcell][kstar] - 1;
		tmp[1][jcell][kstar] = tmp[1][jcell][kstar] + 1;
		flag = 1;
	}
	
	else {
		tmp[0][jcell][kstar] = tmp[0][jcell][kstar] + 1;
		tmp[1][jcell][kstar] = tmp[1][jcell][kstar] - 1;
		flag = 2;
	}
	
	if (tmp[0][jcell][kstar] < 0 and flag ==1) {
		tmp[0][jcell][kstar] = tmp[0][jcell][kstar] + 1;
		tmp[1][jcell][kstar] = tmp[1][jcell][kstar] - 1;
	}
	
	else if (tmp[1][jcell][kstar] < 0 and flag ==2){
		tmp[0][jcell][kstar] = tmp[0][jcell][kstar] - 1;
		tmp[1][jcell][kstar] = tmp[1][jcell][kstar] + 1;
	}
	
	else {
		k =kstar;
		break;
	}

	}
	
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			delete [] tmp[i][j];
		}
		delete [] tmp[i];
	}
	delete [] tmp;
    

	return k;
	
}


/*
int main(int argc, char *argv[]){
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	char filename[200], filename11[200];
	int I, J, K;
	
	
	strcpy(filename, argv[1]);
	sprintf(filename11, "%s.mod",filename);
	ifstream in(filename11);
	if(!in){
		cerr << "Error on reading the input file." << endl;
		exit(0); 
	}
	in >> I >> J >> K;
	
	
	int ***Xnull;
	
	Xnull = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(Xnull)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(Xnull)[i][j] = new int [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				in >> Xnull[i][j][k];
			}
		}
	}
    
	int icell = 0, jcell = 2, k;
	
	srand(time(0));
	irand.seed(time(0));
	k = dist(I, J, icell, jcell, K, Xnull, irand);
	cout << k;
	return k;

}


*/

int OneTable_uni(int I, int J, int K, int ****Xnull, MTRand_int32& irand){
	Matrix2 *cell;
	cell = rmove_uni(J, irand);
//	for (int i = 0; i < 4; i++) {
//	cout << (*cell).rowVectors[0].elements[i] << " " ;
//    }
//	cout << endl;
	int k, t, flag = 0;
	double r1 = 0, r2 = 0, r = 0;
	
	int  **XIJ;
	XIJ = new int *[I];
	for (int i = 0; i < I; i++) {
		XIJ[i] = new int[J]; 
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			XIJ[i][j] = 0;
			for (int k = 0; k < K; k++) {
				XIJ[i][j] = XIJ[i][j] + (*Xnull)[i][j][k];
				
			}
		}
	}
	
	
	int ***tmp;
	
	tmp = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(tmp)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(tmp)[i][j] = new int [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				tmp[i][j][k] = (*Xnull)[i][j][k];
			}
		}
	}
	
	int xmin;
	xmin = maxl_uni(I, J, cell, XIJ);
	
	if (xmin <= 0) {flag = 1;}
	else {
		flag = 0;
		xmin = 1;
		
		
		
		
		
		for (int i = 0; i < 4; i++) {
//			k = dist(I , J, i, (*cell).rowVectors[0].elements[i], K, *Xnull, irand);
//			cout << k << " ";
			k = irand() % K;
			if (i < 2) {
				t = (*cell).rowVectors[0].elements[i];
				(*Xnull)[0][t][k] = (*Xnull)[0][t][k] - 1;
				(*Xnull)[1][t][k] = (*Xnull)[1][t][k] + 1;
				r1 = r1 + log((*Xnull)[1][t][k]);
				r2 = r2 + log((*Xnull)[0][t][k] + 1);
			}
			else {
				t = (*cell).rowVectors[0].elements[i];
				(*Xnull)[0][t][k] = (*Xnull)[0][t][k] + 1;
				(*Xnull)[1][t][k] = (*Xnull)[1][t][k] - 1;
				r1 = r1 + log((*Xnull)[0][t][k]);
				r2 = r2 + log((*Xnull)[1][t][k] + 1);
			}		
		}
//		cout << endl;
		

		for (int i = 0; i < I; i++) 
			for (int j = 0; j < J; j++) 
				for (int k = 0; k < K; k++) 
					if((*Xnull)[i][j][k]<0)flag = 1;
		
			
	

		
		
		
//		for (int i = 0; i < I; i++) {
//			for (int j = 0; j < J; j++) {
//				for (int k = 0; k < K; k++) {
//					cout << tmp[i][j][k]<< " ";
//				}
//				cout << endl;
//			}
//			cout << endl;
//		}
//		cout << endl;
		
//hyper		r = exp(r2 - r1);
//		cout << (1000.0 * r) <<" ";
		
//hyper		int random = irand() % 1000;
//		cout << random << endl;
		
//hyper		if (random > (int)(1000.0 * r)) {flag = 1;}
		
		
		
		if (flag == 1) {
			for (int i = 0; i < I; i++) {
				for (int j = 0; j < J; j++) {
					for (int k = 0; k < K; k++) {
						(*Xnull)[i][j][k] = tmp[i][j][k];
					}
				}
			}
		}
		
		
		
	}
	
//	for (int i = 0; i < I; i++) {
//		for (int j = 0; j < J; j++) {
//			for (int k = 0; k < K; k++) {
//				cout << (*Xnull)[i][j][k] << " ";
//			}
//				cout << endl;
//		}
//			cout << endl;
//	}
	
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			delete [] tmp[i][j];
		}
		delete [] tmp[i];
	}
	delete [] tmp;
	return flag;	
}

/*

 int main(int argc, char *argv[]){
	 unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	 MTRand_int32 irand(init, length);

	 char filename[200], filename11[200];
	 int I, J, K;
	 strcpy(filename, argv[1]);
	 sprintf(filename11, "%s.mod",filename);
	 ifstream in(filename11);
	 if(!in){
		 cerr << "Error on reading the input file." << endl;
		 exit(0); 
	 }
	 in >> I >> J >> K;
	 
	 int ***Xnull;
	 
	 (Xnull) = new int **[I];
	 
	 for (int i = 0; i < I; i++) {
		 (Xnull)[i] = new int *[J];
		 for (int j = 0; j < J; j++) {
			 (Xnull)[i][j] = new int [K];
		 }
	 }
	 
	 for (int i = 0; i < I; i++) {
		 for (int j = 0; j < J; j++) {
			 for (int k = 0; k < K; k++) {
				 in >> (Xnull)[i][j][k];
			 }
		 }
	 }
	 int **XIJ;
	 
	 XIJ = new int *[I];
	 for (int i = 0; i < I; i++) {
		 XIJ[i] = new int[J]; 
	 }
	 
	 for (int i = 0; i < I; i++) {
		 for (int j = 0; j < J; j++) {
			 XIJ[i][j] = 0;
			 for (int k = 0; k < K; k++) {
				 XIJ[i][j] = XIJ[i][j] + (Xnull)[i][j][k];
				 
			 }
		 }
	 }
	 
	 srand(time(0));
	 irand.seed(time(0));
	 
	 int a = OneTable_uni(I, J, K, (&Xnull), irand);
	 cout <<  a;
	 
	 return 0;
	 
 }

	 
	 */
 
 
 
 
  
  
 


double width(int I, int J, int K, Matrix2 *cell, int ***Xnull){
	int tempj, tempk;
	double r1 = 0, r2 = 0, r = 0;
	int ***tmp;
	
    tmp = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(tmp)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(tmp)[i][j] = new int [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				tmp[i][j][k] = (Xnull)[i][j][k];
			}
		}
	}
	
	for (int i = 1; i < 3; i++) {
		tempj = (*cell).rowVectors[i].elements[0];
		tempk = (*cell).rowVectors[i].elements[1];
		tmp[0][tempj][tempk] = tmp[0][tempj][tempk] - 1;
		r2 = r2 + log(tmp[0][tempj][tempk] + 1);
		tmp[1][tempj][tempk] = tmp[1][tempj][tempk] + 1;
		r1 = r1 + log(tmp[1][tempj][tempk]);
	}
	
	
	for (int i = 0; i < 4; i+=3) {
		tempj = (*cell).rowVectors[i].elements[0];
		tempk = (*cell).rowVectors[i].elements[1];
		tmp[1][tempj][tempk] = tmp[1][tempj][tempk] - 1;
		r2 = r2 + log(tmp[1][tempj][tempk] + 1);
		tmp[0][tempj][tempk] = tmp[0][tempj][tempk] + 1;	
		r1 = r1 + log(tmp[0][tempj][tempk]);
	}
	
	r = exp(r2 - r1);
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			delete [] tmp[i][j];
		}
		delete [] tmp[i];
	}
	delete [] tmp;
		
	return r;
		
}

/*
int main(int argc, char *argv[]){
	char filename[200], filename11[200];
	int I, J, K;
	
	strcpy(filename, argv[1]);
	sprintf(filename11, "%s.mod",filename);
	ifstream in(filename11);
	if(!in){
		cerr << "Error on reading the input file." << endl;
		exit(0); 
	}
	in >> I >> J >> K;
	
	int ***Xnull;
	
	Xnull = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(Xnull)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(Xnull)[i][j] = new int [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				in >> Xnull[i][j][k];
			}
		}
	}
	
	
	
	Matrix2 *cell = new Matrix2(4,2);
	(*cell).rowVectors[0].elements[0] = 0;
	(*cell).rowVectors[1].elements[0] = 4;
	(*cell).rowVectors[2].elements[0] = 3;
	(*cell).rowVectors[3].elements[0] = 7;
	(*cell).rowVectors[0].elements[1] = 6;
	(*cell).rowVectors[1].elements[1] = 5;
	(*cell).rowVectors[2].elements[1] = 1;
	(*cell).rowVectors[3].elements[1] = 0;
	
	
	
	
	double r;
	r = width(cell, Xnull);
	cout << r <<endl;
	
	return 0;
	
}*/


int OneTable_bi(int I, int J, int K, int ****Xnull, MTRand_int32& irand){
	Matrix2 *cell;
	cell = rmove_bi(I, J, K, irand);
	
//	for (int i = 0; i < 4; i++) {
//		for (int j = 0; j < 2; j++) {
//			cout << (*cell).rowVectors[i].elements[j] << " " ;
//		}
		
//	}
//	cout << endl;
	
//	int ***tmp;
	int tempj, tempk, flag = 0;
	
//	tmp = new int **[I];
	
//	for (int i = 0; i < I; i++) {
//		(tmp)[i] = new int *[J];
//		for (int j = 0; j < J; j++) {
//			(tmp)[i][j] = new int [K];
//		}
//	}
	
//	for (int i = 0; i < I; i++) {
//		for (int j = 0; j < J; j++) {
//			for (int k = 0; k < K; k++) {
//				tmp[i][j][k] = (*Xnull)[i][j][k];
//			}
//		}
//	}

	
	int xmin;
	xmin = maxl_bi(I, J, K, cell, *Xnull);
	
//	cout << xmin << endl;
	
	if (xmin <= 0) {flag = 1;}

	else {
		flag = 0;
		xmin = 1;
		double r;
		r = width(I, J, K, cell, *Xnull);
		
//		srand(time(0));
		int random = irand() % 1000;
//		cout << random << " " <<(int)(1000.0 * r)<<endl ;
	if (random > (int)(1000.0 * r)){flag = 1;}
 
		
		else {
			
			for (int i = 1; i < 3; i++) {
				tempj = (*cell).rowVectors[i].elements[0];
				tempk = (*cell).rowVectors[i].elements[1];
				(*Xnull)[0][tempj][tempk] = (*Xnull)[0][tempj][tempk] - 1;
				(*Xnull)[1][tempj][tempk] = (*Xnull)[1][tempj][tempk] + 1;
	
			}
			
			
			for (int i = 0; i < 4; i+=3) {
				tempj = (*cell).rowVectors[i].elements[0];
				tempk = (*cell).rowVectors[i].elements[1];
				(*Xnull)[1][tempj][tempk] = (*Xnull)[1][tempj][tempk] - 1;
				(*Xnull)[0][tempj][tempk] = (*Xnull)[0][tempj][tempk] + 1;	
			}
		
			}
		
	}
	
	return flag;	
	
}	

/*
int main(int argc, char *argv[]){
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	
	char filename[200], filename11[200];
	int I, J, K;
	
	strcpy(filename, argv[1]);
	sprintf(filename11, "%s.mod",filename);
	ifstream in(filename11);
	if(!in){
		cerr << "Error on reading the input file." << endl;
		exit(0); 
	}
	in >> I >> J >> K;
	
	int ***Xnull;
	
	(Xnull) = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(Xnull)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(Xnull)[i][j] = new int [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				in >> (Xnull)[i][j][k];
			}
		}
	}
	
	
	srand(time(0));
	irand.seed(time(0));

	
	int a = OneTable_bi(I, J, K, (&Xnull), irand);
	cout <<  a;
	
	return 0;
	
}


*/

double fact(double x){
	double fact = 0.0;
	for(int i = 1; i <= x; i++) fact -= log(((double) i));
	return fact;
}

double probRatio(int I, int J, int K, int ***oldTable, int ***newTable){
	double p = 0.0;
	for (int i = 0; i < I; i++)
		for (int j = 0; j < J; j++)
			for (int k = 0; k < K; k++)
			p += fact(((double)newTable[i][j][k])) - fact((double)oldTable[i][j][k]); 
	return exp(p);
}

int OneTable_indept(int I, int J, int K, int ****Xnull, MTRand_int32& irand){
	Matrix2 *cell;
	cell = rmove_indept(J, K, irand);
	
	int ***tmp;
	int tempj, tempk, flag = 0;
	
		tmp = new int **[I];
	
		for (int i = 0; i < I; i++) {
			(tmp)[i] = new int *[J];
			for (int j = 0; j < J; j++) {
				(tmp)[i][j] = new int [K];
			}
		}
	
		for (int i = 0; i < I; i++) {
			for (int j = 0; j < J; j++) {
				for (int k = 0; k < K; k++) {
					tmp[i][j][k] = (*Xnull)[i][j][k];
				}
			}
		}
	
	
			flag = 0;
	

		

			
			if (irand() % 2 == 0) {
				tempj = (*cell).rowVectors[0].elements[0];
				tempk = (*cell).rowVectors[1].elements[0];
				(*Xnull)[0][tempj][tempk] = (*Xnull)[0][tempj][tempk] + 1;
				(*Xnull)[1][tempj][tempk] = (*Xnull)[1][tempj][tempk] - 1;
				tempj = (*cell).rowVectors[0].elements[1];
				tempk = (*cell).rowVectors[1].elements[1];
				(*Xnull)[0][tempj][tempk] = (*Xnull)[0][tempj][tempk] + 1;
				(*Xnull)[1][tempj][tempk] = (*Xnull)[1][tempj][tempk] - 1;
				tempj = (*cell).rowVectors[0].elements[0];
				tempk = (*cell).rowVectors[1].elements[1];
				(*Xnull)[0][tempj][tempk] = (*Xnull)[0][tempj][tempk] - 1;
				(*Xnull)[1][tempj][tempk] = (*Xnull)[1][tempj][tempk] + 1;
				tempj = (*cell).rowVectors[0].elements[1];
				tempk = (*cell).rowVectors[1].elements[0];
				(*Xnull)[0][tempj][tempk] = (*Xnull)[0][tempj][tempk] - 1;
				(*Xnull)[1][tempj][tempk] = (*Xnull)[1][tempj][tempk] + 1;
				
			}
				else {
					tempj = (*cell).rowVectors[0].elements[0];
					tempk = (*cell).rowVectors[1].elements[0];
					(*Xnull)[0][tempj][tempk] = (*Xnull)[0][tempj][tempk] - 1;
					(*Xnull)[1][tempj][tempk] = (*Xnull)[1][tempj][tempk] + 1;
					tempj = (*cell).rowVectors[0].elements[1];
					tempk = (*cell).rowVectors[1].elements[1];
					(*Xnull)[0][tempj][tempk] = (*Xnull)[0][tempj][tempk] - 1;
					(*Xnull)[1][tempj][tempk] = (*Xnull)[1][tempj][tempk] + 1;
					tempj = (*cell).rowVectors[0].elements[0];
					tempk = (*cell).rowVectors[1].elements[1];
					(*Xnull)[0][tempj][tempk] = (*Xnull)[0][tempj][tempk] + 1;
					(*Xnull)[1][tempj][tempk] = (*Xnull)[1][tempj][tempk] - 1;
					tempj = (*cell).rowVectors[0].elements[1];
					tempk = (*cell).rowVectors[1].elements[0];
					(*Xnull)[0][tempj][tempk] = (*Xnull)[0][tempj][tempk] + 1;
					(*Xnull)[1][tempj][tempk] = (*Xnull)[1][tempj][tempk] - 1;
					
					
				}
				
				

				
for (int i = 0; i < I; i++) 
	for (int j = 0 ; j < J; j++) 
		for (int k = 0; k < K; k++) 
			if ((*Xnull)[i][j][k] < 0) flag = 1;
	

	
	int r = irand() % 1000;
	if(flag != 1)
		if(r > ((int) (1000.0 * probRatio(I, J, K, tmp, (*Xnull))))){
			flag = 1;}
	
	if (flag == 1) {
		for (int i = 0; i < I; i++) 
			for (int j = 0 ; j < J; j++) 
				for (int k = 0; k < K; k++)
					(*Xnull)[i][j][k] = tmp[i][j][k];
	}

	
	for (int i = 0; i < I; i++) {
	    for (int j = 0; j < J; j++) {
			delete [] tmp[i][j];
		}
		delete [] tmp[i];
	}
	delete [] tmp;

									 
				
return flag;	
				
}	
/*
int main(int argc, char *argv[]){
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	
	char filename[200], filename11[200];
	int I, J, K;
	
	strcpy(filename, argv[1]);
	sprintf(filename11, "%s.mod",filename);
	ifstream in(filename11);
	if(!in){
		cerr << "Error on reading the input file." << endl;
		exit(0); 
	}
	in >> I >> J >> K;
	
	int ***Xnull;
	
	(Xnull) = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(Xnull)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(Xnull)[i][j] = new int [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				in >> (Xnull)[i][j][k];
			}
		}
	}
	
	
	srand(time(0));
	irand.seed(time(0));
	
	
	int a = OneTable_indept(I, J, K, (&Xnull), irand);
	cout <<  a;
	
	return 0;
	
}

*/

void read(int argc, char *argv[], int &I, int &J, int &K, int &itr2, int &Skip, int &BurnIn, int ****Xnull){
	char filename[200], filename11[200]; 
	itr2 = atoi(argv[2]); 
	Skip = atoi(argv[3]);
    BurnIn=atoi(argv[4]); 
	strcpy(filename, argv[1]);
	strcpy(filename, argv[1]);
	sprintf(filename11, "%s.mod",filename);
	ifstream in(filename11);
	if(!in){
		cerr << "Error on reading the input file." << endl;
		exit(0); 
	}
//	in >> I >> K >> J;
	in >> I >> J >> K;
	
	(*Xnull) = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(*Xnull)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(*Xnull)[i][j] = new int [K];
		}
	}
	
		for (int i = 0; i < I; i++) 
			for (int j = 0; j < J; j++) 
				for (int k = 0; k < K; k++) 
					in >> (*Xnull)[i][j][k];
	
//	for (int i = 0; i < I; i++) 
//		for (int k = 0; k < K; k++) 
//			for (int j = 0; j < J; j++) 
//				in >> (*Xnull)[i][j][k];
	
	cout << endl << "The input table is:" << endl;
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				cout << (*Xnull)[i][j][k] << " ";
			}
			cout << endl;
		}
		cout << endl;
	}
	cout << endl;
	


	
}

/*
int main(int argc, char *argv[]){
	
	int I, J, K, itr, itr2, Skip, BurnIn, ***Xnull;
	read(argc, argv, I, J, K, itr2, Skip, BurnIn, &Xnull);
	
}*/


/*
int main(int argc, char *argv[]){
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	int I, J, K, itr2, Skip, BurnIn, ***Xnull, freq = 0;
	Vector2d mu1;
	Vector3d mu2;
	double L1, L2, ratio1, ratio;
	
	char outfile1[200], outfile2[200];
	char invocation[200];
	strcpy(invocation, "invocation: ");
	for(int f = 0; f < argc; f++){
		strcat(invocation, argv[f]);
		strcat(invocation, " ");
	}
	cout << invocation << endl; 
	
	read(argc, argv, I, J, K, itr2, Skip, BurnIn, &Xnull);
	
//	cout << itr <<" " << itr2 << " "<< Skip <<  " "<< BurnIn << endl;
	
	strcpy(outfile1, argv[1]);
	strcat(outfile1, ".table");
	strcpy(outfile2, argv[1]);
	strcat(outfile2, ".LRT");
	
	
	ofstream outTable, outLRT;
	outTable.open(outfile1);
	outLRT.open(outfile2);
	
	int **XJK;
	
	XJK = new int *[J];
	for (int j = 0; j < J; j++) {
		XJK[j] = new int[K]; 
	}
	
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			XJK[j][k] = (Xnull)[0][j][k] + (Xnull)[1][j][k];
			//				cout << XJK[j][k] << " ";
		}
		//			cout << endl;
	}
	//		cout << endl;
	
	
	
	int **XIJ;
	
	XIJ = new int *[I];
	for (int i = 0; i < I; i++) {
		XIJ[i] = new int[J]; 
	}
	
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			XIJ[i][j] = 0;
			for (int k = 0; k < K; k++) {
				XIJ[i][j] = XIJ[i][j] + (Xnull)[i][j][k];
				
			}
			//				cout << XIJ[i][j] << " ";
		}
		//			cout << endl;
	}
	//		cout << endl;
	
		
	int *XJ;
	XJ =new int[J];
	
	
	
	for (int j = 0; j < J; j++) {
		XJ[j] = 0;
		for (int i = 0; i < I; i++) {
			for (int k = 0; k < K; k++) {
				XJ[j] = XJ[j] + (Xnull)[i][j][k];				
			}			
		}
		//			cout << XJ[j] << " ";
	}
	//		cout << endl;
	
	
	
	
	
	Matrix2 *stats;
	stats = suff_stat(J, K, Xnull);
	
	cout << endl << "The sufficient statistics under bivariate model is:" << endl;
	
	for (int t = 0; t < 3; t++) {
		cout << (*stats).rowVectors[0].elements[t] << " ";
	} 
	cout << endl;
	
	Matrix2 *ss_null = new Matrix2(1,2);
	
	cout << endl << "The sufficient statistics under univariate model is:" << endl;
	
	for (int i = 0; i < 2; i++) {
		(*ss_null).rowVectors[0].elements[i] = (*stats).rowVectors[0].elements[i];
		cout << (*ss_null).rowVectors[0].elements[i] << " ";
	}
	cout << endl; 	
	
	
	
	mu1 = newton_uni(J, XJ, ss_null);
    mu2 = newton_bi(J, K, XJK, stats);
	cout << "mu1=" << mu1 << " " << "mu2=" << mu2 <<endl;

	L1 = log_uni(J, XIJ, mu1);
	L2 = log_bi(J, K, Xnull, mu2);


	ratio1 = LRT(L1, L2);
	cout << "The LRT of the observed table is:" << ratio1 << endl << endl;
	
	srand(time(0));
	irand.seed(time(0));
	int re_run=0;
	
	for(int numOfItr = 0; numOfItr < BurnIn; numOfItr++){
		re_run = 0;
		while(OneTable_uni(I, J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 100) break;
		}
	}
	
	for(int numOfItr = 0; numOfItr < itr2*Skip; numOfItr++){
		re_run = 0;
		while(OneTable_uni(I, J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 100000) break;
		}
		
		
		
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				XJK[j][k] = (Xnull)[0][j][k] + (Xnull)[1][j][k];
//				cout << XJK[j][k] << " ";
			}
//			cout << endl;
		}
//		cout << endl;
		
		stats = suff_stat(J, K, Xnull);

		
		
		mu1 = newton_uni(J, XJ, ss_null);
		mu2 = newton_bi(J, K, XJK, stats);
		
		if (mu1[0] == -10000 & mu1[1] == -10000 & mu2[0] == -10000 & mu2[1] == -10000 & mu2[2] == -10000 ) {
			itr2 = itr2 - 1;
		}
		
		else {
			L1 = log_uni(J, XIJ, mu1);
			L2 = log_bi(J, K, Xnull, mu2);
			
			
			ratio = LRT(L1, L2);
			
			if((numOfItr % Skip) == 0){
				for(int i = 0; i < I; i++){
					for(int j = 0; j < J; j++){
						for (int k = 0; k < K; k++)
							outTable << Xnull[i][j][k] << " ";
						outTable << endl;
					}outTable << endl;
				}outTable << endl;
			}
			
			if((numOfItr % Skip) == 0){
				outLRT << ratio << endl;
				
				if(ratio1 <= ratio) freq++;
			}outLRT ; 	 
		}

		
		
	}
	
	cout << "p-value is: " << ((double) freq)/ ((double)itr2) << endl << endl;
	return 0;
	
}*/
 

/*
int main(int argc, char *argv[]){
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	int I, J, K, itr2, Skip, BurnIn, ***Xnull, freq = 0;
	Vector3d mu2;
	double ***p, L3, L2, ratio2, ratio;
	
	char outfile1[200], outfile2[200];
	char invocation[200];
	strcpy(invocation, "invocation: ");
	for(int f = 0; f < argc; f++){
		strcat(invocation, argv[f]);
		strcat(invocation, " ");
	}
	cout << invocation << endl; 
	
	read(argc, argv, I, J, K, itr2, Skip, BurnIn, &Xnull);
	
	//	cout << itr <<" " << itr2 << " "<< Skip <<  " "<< BurnIn << endl;
	
	strcpy(outfile1, argv[1]);
	strcat(outfile1, ".table");
	strcpy(outfile2, argv[1]);
	strcat(outfile2, ".LRT");
	
	
	ofstream outTable, outLRT;
	outTable.open(outfile1);
	outLRT.open(outfile2);
	
	p = new double **[I];
	
	for (int i = 0; i < I; i++) {
		(p)[i] = new double *[J];
		for (int j = 0; j < J; j++) {
			(p)[i][j] = new double [K];
		}
	}
	
	int **XJK;
	
	XJK = new int *[J];
	for (int j = 0; j < J; j++) {
		XJK[j] = new int[K]; 
	}
	
	cout << endl << "The XJK table is:" << endl;
	
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			XJK[j][k] = (Xnull)[0][j][k] + (Xnull)[1][j][k];
			cout << XJK[j][k] << " ";
		}
		cout << endl;
	}
	cout << endl;
	
	
	
	
	Matrix2 *stats;
	stats = suff_stat(J, K, Xnull);
	
	cout << endl << "The sufficient statistics under bivariate model is:" << endl;
	
	for (int t = 0; t < 3; t++) {
		cout << (*stats).rowVectors[0].elements[t] << " ";
	} 
	cout << endl;
	

    mu2 = newton_bi(J, K, XJK, stats);
	no3way_Indept_IPF(I, J, K, Xnull, &p);
	
	
	L3 = log_indept(J, K, Xnull, p);
	cout << L3 << " ";
	L2 = log_bi(J, K, Xnull, mu2);
	cout << L2 << endl;
	
	ratio2 = LRT(L2, L3);
	cout << "The LRT of the observed table is:" << ratio2 << endl << endl;
	
	srand(time(0));
	irand.seed(time(0));
	int re_run=0;
	
	for(int numOfItr = 0; numOfItr < BurnIn; numOfItr++){
		re_run = 0;
		while(OneTable_bi(I, J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 100) break;
		}
	}
	
	for(int numOfItr = 0; numOfItr < itr2*Skip; numOfItr++){
		re_run = 0;
		while(OneTable_bi(I, J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 100) break;
		}
		no3way_Indept_IPF(I, J, K, Xnull, &p);
		mu2 = newton_bi(J, K, XJK, stats);
		
		L3 = log_indept(J, K, Xnull, p);
		L2 = log_bi(J, K, Xnull, mu2);
		
		ratio = LRT(L2, L3);
		
		if((numOfItr % Skip) == 0){
			for(int i = 0; i < I; i++){
				for(int j = 0; j < J; j++){
					for (int k = 0; k < K; k++)
						outTable << Xnull[i][j][k] << " ";
				    outTable << endl;
			    }outTable << endl;
		    }outTable << endl;
		}
		
		if((numOfItr % Skip) == 0){
			outLRT << ratio << endl;
			
			if(ratio2 <= ratio) freq++;
		}outLRT ; 	 
	}
	
	cout << "p-value is: " << ((double) freq)/ ((double)itr2) << endl << endl;
	return 0;
	
}*/

int main(int argc, char *argv[]){
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	int I, J, K, itr2, Skip, BurnIn, ***Xnull, freq = 0;

	double ***p, chisq0, chisq;
	
	char outfile1[200], outfile2[200];
	char invocation[200];
	strcpy(invocation, "invocation: ");
	for(int f = 0; f < argc; f++){
		strcat(invocation, argv[f]);
		strcat(invocation, " ");
	}
	cout << invocation << endl; 
	
	read(argc, argv, I, J, K, itr2, Skip, BurnIn, &Xnull);
	
	//	cout << itr <<" " << itr2 << " "<< Skip <<  " "<< BurnIn << endl;
	
	strcpy(outfile1, argv[1]);
	strcat(outfile1, ".table");
	strcpy(outfile2, argv[1]);
	strcat(outfile2, ".LRT");
	
	
	ofstream outTable, outLRT;
	outTable.open(outfile1);
	outLRT.open(outfile2);
	
	p = new double **[I];
	
	for (int i = 0; i < I; i++) {
		(p)[i] = new double *[J];
		for (int j = 0; j < J; j++) {
			(p)[i][j] = new double [K];
		}
	}

	
	int **XJK;
	
	XJK = new int *[J];
	for (int j = 0; j < J; j++) {
		XJK[j] = new int[K]; 
	}
	
	cout << endl << "The XJK table is:" << endl;
	
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			XJK[j][k] = (Xnull)[0][j][k] + (Xnull)[1][j][k];
			cout << XJK[j][k] << " ";
		}
		cout << endl;
	}
	cout << endl;
	
	
	
	
	Matrix2 *stats;
	stats = suff_stat(J, K, Xnull);
	
	cout << endl << "The sufficient statistics under bivariate model is:" << endl;
	
	for (int t = 0; t < 3; t++) {
		cout << (*stats).rowVectors[0].elements[t] << " ";
	} 
	cout << endl;
	
//	Vector3d mu2;
//  mu2 = newton_bi(J, K, XJK, stats);
	

	no3way_Indept_IPF(I, J, K, Xnull, &p);
	
	
	chisq0 = chisq_stat(I, J, K, Xnull, p);
	cout << "The Pearson's Chi-Square statistics of the observed table is:" << chisq0 << endl << endl;
	
	srand(time(0));
	irand.seed(time(0));
	int re_run=0;
	
	for(int numOfItr = 0; numOfItr < BurnIn; numOfItr++){
		re_run = 0;
		while(OneTable_indept(I, J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 100) break;
		}
	}
	
	for(int numOfItr = 0; numOfItr < itr2*Skip; numOfItr++){
		re_run = 0;
		while(OneTable_indept(I, J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 100) break;
		}
		no3way_Indept_IPF(I, J, K, Xnull, &p);
		
		chisq = chisq_stat(I, J, K, Xnull, p);
		
//		mu2 = newton_bi(J, K, XJK, stats);
		
//		chisq = chisq_stat(I, J, K, Xnull, mu2);
	
		
		if((numOfItr % Skip) == 0){
			for(int i = 0; i < I; i++){
				for(int j = 0; j < J; j++){
					for (int k = 0; k < K; k++)
						outTable << Xnull[i][j][k] << " ";
				    outTable << endl;
			    }outTable << endl;
		    }outTable << endl;
		}
		
		if((numOfItr % Skip) == 0){
			outLRT << chisq<< endl;
			
			if(chisq0 <= chisq) freq++;
		}outLRT ; 	 
	}
	
	cout << "p-value is: " << ((double) freq)/ ((double)itr2) << endl << endl;
	return 0;
	
}

