#include <fstream.h>
#include <iostream.h>
#include <cstdlib>
#include <cstring>
#include <math.h>
#include <algorithm>
#include <time.h>
#include "Matrix.h"
#include <Eigen>
#include "mtrand.h"

using namespace std;
using namespace Eigen;


unsigned long MTRand_int32::state[n] = {0x0UL};
int MTRand_int32::p = 0;
bool MTRand_int32::init = false;

void MTRand_int32::gen_state() { // generate new state vector
	for (int i = 0; i < (n - m); ++i)
		state[i] = state[i + m] ^ twiddle(state[i], state[i + 1]);
	for (int i = n - m; i < (n - 1); ++i)
		state[i] = state[i + m - n] ^ twiddle(state[i], state[i + 1]);
	state[n - 1] = state[m - 1] ^ twiddle(state[n - 1], state[0]);
	p = 0; // reset position
}

void MTRand_int32::seed(unsigned long s) {  // init by 32 bit seed
	state[0] = s & 0xFFFFFFFFUL; // for > 32 bit machines
	for (int i = 1; i < n; ++i) {
		state[i] = 1812433253UL * (state[i - 1] ^ (state[i - 1] >> 30)) + i;
		// see Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier
		// in the previous versions, MSBs of the seed affect only MSBs of the array state
		// 2002/01/09 modified by Makoto Matsumoto
		state[i] &= 0xFFFFFFFFUL; // for > 32 bit machines
	}
	p = n; // force gen_state() to be called for next random number
}

void MTRand_int32::seed(const unsigned long* array, int size) { // init by array
	seed(19650218UL);
	int i = 1, j = 0;
	for (int k = ((n > size) ? n : size); k; --k) {
		state[i] = (state[i] ^ ((state[i - 1] ^ (state[i - 1] >> 30)) * 1664525UL))
		+ array[j] + j; // non linear
		state[i] &= 0xFFFFFFFFUL; // for > 32 bit machines
		++j; j %= size;
		if ((++i) == n) { state[0] = state[n - 1]; i = 1; }
	}
	for (int k = n - 1; k; --k) {
		state[i] = (state[i] ^ ((state[i - 1] ^ (state[i - 1] >> 30)) * 1566083941UL)) - i;
		state[i] &= 0xFFFFFFFFUL; // for > 32 bit machines
		if ((++i) == n) { state[0] = state[n - 1]; i = 1; }
	}
	state[0] = 0x80000000UL; // MSB is 1; assuring non-zero initial array
	p = n; // force gen_state() to be called for next random number
}




void no3way_Indept_IPF(int R, int C, int** Xnull, double*** p){
	int Loop, itr = 100;
	int  Rsum[R], Csum[C];
	double tmp[R][C], Xold[R][C];
	double oldRsum[R], oldCsum[C];
	
	
	for (int i=0; i<R; i++){ 
		Rsum[i]=0;
		for (int j=0; j<C; j++){ 
			Rsum[i]+=Xnull[i][j];
		}
	}	
	
	
	
	for (int j=0; j<C; j++){ 
		Csum[j]=0;
		for (int i=0; i<R; i++){ 
			Csum[j]+=Xnull[i][j];
        }
	}
	
	for(int i = 0; i < R; i++){
		for(int j = 0; j < C; j++){
			Xold[i][j] = 1.0;
		}
	}
	
	Loop=1;
	while (Loop <= itr) {
		for (int i=0; i<R; i++){ 
			oldRsum[i]=0.0;
			for (int j=0; j<C; j++){ 
				oldRsum[i]+=Xold[i][j];
			}
		}	
		
		for(int i = 0; i < R; i++){
			for(int j = 0; j < C; j++){
				if(oldRsum[i] > 0){
					tmp[i][j] = Xold[i][j] * ((double)Rsum[i]) / oldRsum[i];					
				}
			}
		}
		
		for(int i = 0; i < R; i++){
			for(int j = 0; j < C; j++){
				Xold[i][j] = tmp[i][j];
			}
		}
		
		for (int j=0; j<C; j++){ 
			oldCsum[j]=0.0;
			for (int i=0; i<R; i++){ 
				oldCsum[j]+=Xold[i][j];
			}
			
		}
		
		for(int i = 0; i < R; i++){
			for(int j = 0; j < R; j++){
				if(oldCsum[j] > 0){
					tmp[i][j] = Xold[i][j] * ((double)Csum[j]) / oldCsum[j];
				}
				
			}
			
		}
		
		for(int i = 0; i < R; i++){
			for(int j = 0; j < C; j++){
				Xold[i][j] = tmp[i][j];
			}
		}
		
		Loop++;
	}
	
	for(int i = 0; i < R; i++){
		for(int j = 0; j < C; j++){
			(*p)[i][j] = Xold[i][j];
		}
	}
}






double chisq_stat(int J, int K, int **Xnull, double **p){
	double chisq;
	
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				
				if (p[j][k]>0) {
					chisq += (Xnull[j][k] - p[j][k]) * (Xnull[j][k] - p[j][k]) / p[j][k];
				}
			}
		}

	return chisq;
}

static void getTwo (int *dst, int num, MTRand_int32& irand) {
    int sz, pos, i, src[num];
    for (i = 0; i < sizeof(src)/sizeof(*src); i++)
        src[i] = i;
    sz = num;
    for (i = 0; i < 2; i++) {
        pos = irand() % sz;
        dst[i] = src[pos];
        src[pos] = src[sz-1];
        sz--;
    }
}

Matrix2 *rmove_indept(int J, int K, MTRand_int32& irand){
	Matrix2 *jk = new Matrix2(2,2);
	
	int numbers1[2], numbers2[2];
    getTwo (numbers1, J, irand);
	
//	cout <<"J is"<< numbers1[0]<<" "<<numbers1[1]<<endl;
	
	(*jk).rowVectors[0].elements[0] = numbers1[0];
	(*jk).rowVectors[0].elements[1] = numbers1[1];
	

    getTwo (numbers2, K, irand);
	(*jk).rowVectors[1].elements[0] = numbers2[0];
	(*jk).rowVectors[1].elements[1] = numbers2[1];
	
//	cout << "K is"<<numbers2[0]<<" "<<numbers2[1]<<endl;
	
//	for (int i = 0; i < 2; i++) {
//		for (int j = 0; j < 2; j++) {
//			cout << (*jk).rowVectors[i].elements[j] << " " ;
//		}
		
//	}
	return jk;
	
}



double fact(double x){
	double fact = 0.0;
	for(int i = 1; i <= x; i++) fact -= log(((double) i));
	return fact;
}

double probRatio(int J, int K, int **oldTable, int **newTable){
	double p = 0.0;
		for (int j = 0; j < J; j++)
			for (int k = 0; k < K; k++)
			p += fact(((double)newTable[j][k])) - fact((double)oldTable[j][k]); 
	return exp(p);
}

int OneTable_indept(int J, int K, int ***Xnull, MTRand_int32& irand){
	Matrix2 *cell;
	cell = rmove_indept(J, K, irand);
	
	int **tmp;
	int tempj, tempk, flag = 0;
	
	tmp = new int *[J];
	for (int j = 0; j < J; j++) {
		tmp[j] = new int[K]; 
	}
	

			for (int j = 0; j < J; j++) {
				for (int k = 0; k < K; k++) {
					tmp[j][k] = (*Xnull)[j][k];
				}
			}
		
	
	
			flag = 0;
	

		

			
			if (irand() % 2 == 0) {
				tempj = (*cell).rowVectors[0].elements[0];
				tempk = (*cell).rowVectors[1].elements[0];
				(*Xnull)[tempj][tempk] = (*Xnull)[tempj][tempk] + 1;
				tempj = (*cell).rowVectors[0].elements[1];
				tempk = (*cell).rowVectors[1].elements[1];
				(*Xnull)[tempj][tempk] = (*Xnull)[tempj][tempk] + 1;
				tempj = (*cell).rowVectors[0].elements[0];
				tempk = (*cell).rowVectors[1].elements[1];
				(*Xnull)[tempj][tempk] = (*Xnull)[tempj][tempk] - 1;
				tempj = (*cell).rowVectors[0].elements[1];
				tempk = (*cell).rowVectors[1].elements[0];
				(*Xnull)[tempj][tempk] = (*Xnull)[tempj][tempk] - 1;
				
			}
				else {
					tempj = (*cell).rowVectors[0].elements[0];
					tempk = (*cell).rowVectors[1].elements[0];
					(*Xnull)[tempj][tempk] = (*Xnull)[tempj][tempk] - 1;
					tempj = (*cell).rowVectors[0].elements[1];
					tempk = (*cell).rowVectors[1].elements[1];
					(*Xnull)[tempj][tempk] = (*Xnull)[tempj][tempk] - 1;
					tempj = (*cell).rowVectors[0].elements[0];
					tempk = (*cell).rowVectors[1].elements[1];
					(*Xnull)[tempj][tempk] = (*Xnull)[tempj][tempk] + 1;
					tempj = (*cell).rowVectors[0].elements[1];
					tempk = (*cell).rowVectors[1].elements[0];
					(*Xnull)[tempj][tempk] = (*Xnull)[tempj][tempk] + 1;
					
					
				}
				
				

				

	for (int j = 0 ; j < J; j++) 
		for (int k = 0; k < K; k++) 
			if ((*Xnull)[j][k] < 0) flag = 1;
	

	
//	int r = irand() % 1000;
//	if(flag != 1)
//		if(r > ((int) (1000.0 * probRatio(J, K, tmp, (*Xnull))))){
//			flag = 1;}
	
	if (flag == 1) {
			for (int j = 0 ; j < J; j++) 
				for (int k = 0; k < K; k++)
					(*Xnull)[j][k] = tmp[j][k];
	}

	
	    for (int j = 0; j < J; j++) 
			delete [] tmp[j];

	
	delete [] tmp;

									 
				
return flag;	
				
}	
/*
int main(int argc, char *argv[]){
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	
	char filename[200], filename11[200];
	int I, J, K;
	
	strcpy(filename, argv[1]);
	sprintf(filename11, "%s.mod",filename);
	ifstream in(filename11);
	if(!in){
		cerr << "Error on reading the input file." << endl;
		exit(0); 
	}
	in >> I >> J >> K;
	
	int ***Xnull;
	
	(Xnull) = new int **[I];
	
	for (int i = 0; i < I; i++) {
		(Xnull)[i] = new int *[J];
		for (int j = 0; j < J; j++) {
			(Xnull)[i][j] = new int [K];
		}
	}
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				in >> (Xnull)[i][j][k];
			}
		}
	}
	
	
	srand(time(0));
	irand.seed(time(0));
	
	
	int a = OneTable_indept(I, J, K, (&Xnull), irand);
	cout <<  a;
	
	return 0;
	
}

*/

void read(int argc, char *argv[], int &J, int &K, int &itr2, int &Skip, int &BurnIn, int ***Xnull){
	char filename[200], filename11[200]; 
	itr2 = atoi(argv[2]); 
	Skip = atoi(argv[3]);
    BurnIn=atoi(argv[4]); 
	strcpy(filename, argv[1]);
	strcpy(filename, argv[1]);
	sprintf(filename11, "%s.mod",filename);
	ifstream in(filename11);
	if(!in){
		cerr << "Error on reading the input file." << endl;
		exit(0); 
	}
//	in >> I >> K >> J;
	in >> J >> K;
	
	(*Xnull) = new int *[J];
	

		for (int j = 0; j < J; j++) {
			(*Xnull)[j] = new int [K];
		}
	
	

			for (int j = 0; j < J; j++) 
				for (int k = 0; k < K; k++) 
					in >> (*Xnull)[j][k];
	
//	for (int i = 0; i < I; i++) 
//		for (int k = 0; k < K; k++) 
//			for (int j = 0; j < J; j++) 
//				in >> (*Xnull)[i][j][k];
	
	cout << endl << "The input table is:" << endl;
	

		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				cout << (*Xnull)[j][k] << " ";
			}
			cout << endl;
		}
		cout << endl;
	
	


	
}


int main(int argc, char *argv[]){
	clock_t t1, t2;
	t1 = clock();
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	int J, K, itr2, Skip, BurnIn, **Xnull, freq = 0;

	double **p, chisq0, chisq;
	
	char outfile1[200], outfile2[200];
	char invocation[200];
	strcpy(invocation, "invocation: ");
	for(int f = 0; f < argc; f++){
		strcat(invocation, argv[f]);
		strcat(invocation, " ");
	}
	cout << invocation << endl; 
	
	read(argc, argv, J, K, itr2, Skip, BurnIn, &Xnull);
	
	//	cout << itr <<" " << itr2 << " "<< Skip <<  " "<< BurnIn << endl;
	
	strcpy(outfile1, argv[1]);
	strcat(outfile1, ".table");
	strcpy(outfile2, argv[1]);
	strcat(outfile2, ".LRT");
	
	
	ofstream outTable, outLRT;
	outTable.open(outfile1);
	outLRT.open(outfile2);
	
	p = new double *[J];
	
		for (int j = 0; j < J; j++) {
			(p)[j] = new double [K];
		}
	

	

	

	

	no3way_Indept_IPF(J, K, Xnull, &p);
	
	
	chisq0 = chisq_stat(J, K, Xnull, p);
	cout << "The Pearson's Chi-Square statistics of the observed table is:" << chisq0 << endl << endl;
	
	srand(time(0));
	irand.seed(time(0));
	int re_run=0;
	
	for(int numOfItr = 0; numOfItr < BurnIn; numOfItr++){
		re_run = 0;
		while(OneTable_indept(J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 100) break;
		}
	}
	
	for(int numOfItr = 0; numOfItr < itr2*Skip; numOfItr++){
		re_run = 0;
		while(OneTable_indept(J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 100) break;
		}
		no3way_Indept_IPF(J, K, Xnull, &p);
		
		chisq = chisq_stat(J, K, Xnull, p);
		

	
		
		if((numOfItr % Skip) == 0){
			
				for(int j = 0; j < J; j++){
					for (int k = 0; k < K; k++)
						outTable << Xnull[j][k] << " ";
				    outTable << endl;
			    }outTable << endl;
	
		}
		
		if((numOfItr % Skip) == 0){
			outLRT << chisq<< endl;
			
			if(chisq0 <= chisq) freq++;
		}outLRT ; 	 
	}
	
	cout << "p-value is: " << ((double) freq)/ ((double)itr2) << endl << endl;
	t2 = clock();
	cout << "Running time is " << (float(t2) - float(t1))/CLOCKS_PER_SEC;
	return 0;
	
}

