/*
 *  move.cpp
 *  
 *
 *  Created by Feng Zhou on 6/29/11.
 *  Copyright 2011 University of Kentucky. All rights reserved.
 *
 */

#include <fstream.h>
#include <iostream.h>
#include <cstdlib>
#include <cstring>
#include <math.h>
#include <algorithm>
#include <time.h>
#include "Matrix.h"
#include <Eigen>
#include "mtrand.h"

#include "move.h"

using namespace std;
using namespace Eigen;


void swap(int x, int y){
	int tmp;
	tmp = x;
	x = y;
	y = tmp;
}

void bubblesort(int n, Matrix2 *cell) {
	for (int i = 0; i < n-1; i++)
		for (int j = n-1; j > i; --j)
			if ((*cell).rowVectors[0].elements[j] < (*cell).rowVectors[0].elements[j-1])
				swap((*cell).rowVectors[0].elements[j] , (*cell).rowVectors[0].elements[j-1]);
}

void intsort(int size, int a[]) {
	int hold;
	for (int pass = 1; pass < size; pass++) {
		for (int j = 0; j < size - 1; j++) {
			if (a[j] > a[j + 1]) {
				hold = a[j];
				a[j] = a[j + 1];
				a[j + 1] = hold;
			}
		}
	}
	
}

static void getThree (int *dst, int J, MTRand_int32& irand) {
    int sz, pos, i, src[J];
    for (i = 0; i < sizeof(src)/sizeof(*src); i++)
        src[i] = i;
    sz = J;
    for (i = 0; i < 3; i++) {
        pos = irand() % sz;
        dst[i] = src[pos];
        src[pos] = src[sz-1];
        sz--;
    }
}

Matrix2 *rmove_uni(int J, MTRand_int32& irand){
	int i = 0, temp;
    Matrix2 *cell = new Matrix2(1,4);
	Matrix2 *jcell = new Matrix2(1,4);
	
    
	int numbers[3];
    getThree (numbers, J, irand);
	
	intsort(3, numbers);
	
	for (int i = 0; i < 3; i++) {
		(*cell).rowVectors[0].elements[i] = numbers[i];
		//		cout << (*cell).rowVectors[0].elements[i] << " ";
	}
	//	cout << endl;
	
	
	
	
	if ((*cell).rowVectors[0].elements[1] - (*cell).rowVectors[0].elements[0] <= (*cell).rowVectors[0].elements[2] - (*cell).rowVectors[0].elements[1]) {
		(*cell).rowVectors[0].elements[3] = (*cell).rowVectors[0].elements[2] - ((*cell).rowVectors[0].elements[1] - (*cell).rowVectors[0].elements[0]);
		//	    cout << (*cell).rowVectors[0].elements[0] << " "<<(*cell).rowVectors[0].elements[1]<< " "<<(*cell).rowVectors[0].elements[2]<< " "<<(*cell).rowVectors[0].elements[3]<<endl;
	}
	
	else {
		(*cell).rowVectors[0].elements[3] = (*cell).rowVectors[0].elements[0] + ((*cell).rowVectors[0].elements[2] - (*cell).rowVectors[0].elements[1]);
		//      cout << (*cell).rowVectors[0].elements[0] << " "<<(*cell).rowVectors[0].elements[1]<< " "<<(*cell).rowVectors[0].elements[2]<< " "<<(*cell).rowVectors[0].elements[3]<<endl;
	}
	
	bubblesort(4, cell);
	
	if (irand() % 2 == 0) {
		(*jcell).rowVectors[0].elements[0] = (*cell).rowVectors[0].elements[0];
		(*jcell).rowVectors[0].elements[1] = (*cell).rowVectors[0].elements[3];
		(*jcell).rowVectors[0].elements[2] = (*cell).rowVectors[0].elements[1];
		(*jcell).rowVectors[0].elements[3] = (*cell).rowVectors[0].elements[2];
	}
	
	else {
		(*jcell).rowVectors[0].elements[0] = (*cell).rowVectors[0].elements[1];
		(*jcell).rowVectors[0].elements[1] = (*cell).rowVectors[0].elements[2];
		(*jcell).rowVectors[0].elements[2] = (*cell).rowVectors[0].elements[0];
		(*jcell).rowVectors[0].elements[3] = (*cell).rowVectors[0].elements[3];
	}
    
	//	for (int i = 0; i < 4; i++) {
	//		cout << (*jcell).rowVectors[0].elements[i] << " ";
	//	}
	//	cout << endl; 
	
	return jcell;
	
}

/*
 int main(){
 unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
 MTRand_int32 irand(init, length);
 srand(time(0));
 irand.seed(time(0));
 int t=0;
 int J = 8;
 Matrix2 *cell;
 cell = rmove_uni(J, irand);
 
 cout << endl;
 for (int i = 0; i < 4; i++) {
 cout << (*cell).rowVectors[0].elements[i] << " " ;
 }
 
 return 0;	
 
 }*/

static void getTwo (int *dst, int num, MTRand_int32& irand) {
    int sz, pos, i, src[num];
    for (i = 0; i < sizeof(src)/sizeof(*src); i++)
        src[i] = i;
    sz = num;
    for (i = 0; i < 2; i++) {
        pos = irand() % sz;
        dst[i] = src[pos];
        src[pos] = src[sz-1];
        sz--;
    }
}

Matrix2 *rmove_bi(int I, int J, int K, MTRand_int32& irand){
	int jstar, kstar, j1, j2, k1, k2, j[2], k[2];
	jstar = 1 + irand() % J;
	//	cout << jstar << endl;
	Matrix2 *jk = new Matrix2(4,2);
	
	if (jstar == J) {kstar = 1 + (irand() % (K - 1));}
	else if (jstar == 1) {kstar = 2 + (irand() % (K - 1));}
	else {kstar = 1 + (irand() % K);}
	//	cout << kstar<<endl;
	int numbers[2];
    getTwo (numbers, jstar*kstar, irand);
	
	numbers[0] = numbers[0] + 1;
	numbers[1] = numbers[1] + 1;
	
	//	cout << numbers[0]<< " "<<numbers[1]<<endl;
	
	if (numbers[0] % kstar == 0) {
		j1 = (numbers[0] - numbers[0] % kstar) / kstar;
		k1 = kstar;
	}
	else {
		j1 = (numbers[0] - numbers[0] % kstar) / kstar + 1;
		k1 = numbers[0] % kstar;
	}
	
	if (numbers[1] % kstar == 0) {
		j2 = (numbers[1] - numbers[1] % kstar) / kstar;
		k2 = kstar;
	}
	else {
		j2 = (numbers[1] - numbers[1] % kstar) / kstar + 1;
		k2 = numbers[1] % kstar;
	}
	
	j[0] = j1;
	j[1] = j2;
	k[0] = k1;
	k[1] = k2;
	
	j1 = J - jstar;
	k1 = K - kstar;
	
	if (irand() % 2 == 0) {
		(*jk).rowVectors[0].elements[0] = j[0] - 1;
		(*jk).rowVectors[0].elements[1] = k[0] - 1;
		(*jk).rowVectors[1].elements[0] = j[1] - 1;
		(*jk).rowVectors[1].elements[1] = k[1] - 1;
		(*jk).rowVectors[2].elements[0] = j[0] + j1 - 1;
		(*jk).rowVectors[2].elements[1] = k[0] + k1 - 1;
		(*jk).rowVectors[3].elements[0] = j[1] + j1 - 1;
		(*jk).rowVectors[3].elements[1] = k[1] + k1 - 1;
	}
	else {
		(*jk).rowVectors[0].elements[0] = j[0] - 1;
		(*jk).rowVectors[0].elements[1] = K - k[0];
		(*jk).rowVectors[1].elements[0] = j[1] - 1;
		(*jk).rowVectors[1].elements[1] = K - k[1];
		(*jk).rowVectors[2].elements[0] = j[0] + j1 - 1;
		(*jk).rowVectors[2].elements[1] = K - k[0] - k1;
		(*jk).rowVectors[3].elements[0] = j[1] + j1 - 1;
		(*jk).rowVectors[3].elements[1] = K - k[1] - k1;
	}
	
	
	return jk;
	
	
}


Matrix2 *rmove_indept(int J, int K, MTRand_int32& irand){
	Matrix2 *jk = new Matrix2(2,2);
	
	int numbers1[2], numbers2[2];
	
    getTwo (numbers1, J, irand);	
	(*jk).rowVectors[0].elements[0] = numbers1[0];
	(*jk).rowVectors[0].elements[1] = numbers1[1];
	
	
    getTwo (numbers2, K, irand);
	(*jk).rowVectors[1].elements[0] = numbers2[0];
	(*jk).rowVectors[1].elements[1] = numbers2[1];
	
	
	//	for (int i = 0; i < 2; i++) {
	//		for (int j = 0; j < 2; j++) {
	//			cout << (*jk).rowVectors[i].elements[j] << " " ;
	//		}
	
	//	}
	
	return jk;
	
}
