/*
 *  univariate.cpp
 *  
 *
 *  Created by Feng Zhou on 6/29/11.
 *  Copyright 2011 University of Kentucky. All rights reserved.
 *
 */

#include <fstream.h>
#include <iostream.h>
#include <cstdlib>
#include <cstring>
#include <math.h>
#include <algorithm>
#include <time.h>
#include "Matrix.h"
#include <Eigen>
#include "mtrand.h"
#include "suffstat.h"
#include "MLE.h"
#include "LRT.h"
#include "move.h"
#include "length.h"
#include "onetable.h"
#include "read.h"

using namespace std;
using namespace Eigen;



int main(int argc, char *argv[]){
	clock_t t1, t2;
	t1 = clock();
	unsigned long init[4] = {0x123, 0x234, 0x345, 0x456}, length = 4;
	MTRand_int32 irand(init, length);
	int I, J, K, itr2, Skip, BurnIn, ***Xnull, freq = 0;
	Vector2d mu1;
	Vector3d mu2;
	double L1, L2, ratio1, ratio;
	
	char outfile1[200], outfile2[200];
	char invocation[200];
	strcpy(invocation, "invocation: ");
	for(int f = 0; f < argc; f++){
		strcat(invocation, argv[f]);
		strcat(invocation, " ");
	}
	cout << invocation << endl; 
	
	read(argc, argv, I, J, K, itr2, Skip, BurnIn, &Xnull);
	
	//	cout << itr <<" " << itr2 << " "<< Skip <<  " "<< BurnIn << endl;
	
	strcpy(outfile1, argv[1]);
	strcat(outfile1, ".table");
	strcpy(outfile2, argv[1]);
	strcat(outfile2, ".LRT");
	
	
	ofstream outTable, outLRT;
	outTable.open(outfile1);
	outLRT.open(outfile2);
	
	int **XJK;
	
	XJK = new int *[J];
	for (int j = 0; j < J; j++) {
		XJK[j] = new int[K]; 
	}
	
	for (int j = 0; j < J; j++) {
		for (int k = 0; k < K; k++) {
			XJK[j][k] = (Xnull)[0][j][k] + (Xnull)[1][j][k];
			//				cout << XJK[j][k] << " ";
		}
		//			cout << endl;
	}
	//		cout << endl;
	
	
	
	int **XIJ;
	
	XIJ = new int *[I];
	for (int i = 0; i < I; i++) {
		XIJ[i] = new int[J]; 
	}
	
	
	for (int i = 0; i < I; i++) {
		for (int j = 0; j < J; j++) {
			XIJ[i][j] = 0;
			for (int k = 0; k < K; k++) {
				XIJ[i][j] = XIJ[i][j] + (Xnull)[i][j][k];
				
			}
			//				cout << XIJ[i][j] << " ";
		}
		//			cout << endl;
	}
	//		cout << endl;
	
	
	int *XJ;
	XJ =new int[J];
	
	
	
	for (int j = 0; j < J; j++) {
		XJ[j] = 0;
		for (int i = 0; i < I; i++) {
			for (int k = 0; k < K; k++) {
				XJ[j] = XJ[j] + (Xnull)[i][j][k];				
			}			
		}
		//			cout << XJ[j] << " ";
	}
	//		cout << endl;
	
	
	
	
	
	Matrix2 *stats;
	stats = suff_stat(J, K, Xnull);
	
	cout << endl << "The sufficient statistics under bivariate model is:" << endl;
	
	for (int t = 0; t < 3; t++) {
		cout << (*stats).rowVectors[0].elements[t] << " ";
	} 
	cout << endl;
	
	Matrix2 *ss_null = new Matrix2(1,2);
	
	cout << endl << "The sufficient statistics under univariate model is:" << endl;
	
	for (int i = 0; i < 2; i++) {
		(*ss_null).rowVectors[0].elements[i] = (*stats).rowVectors[0].elements[i];
		cout << (*ss_null).rowVectors[0].elements[i] << " ";
	}
	cout << endl; 	
	
	
	
	mu1 = newton_uni(J, XJ, ss_null);
    mu2 = newton_bi(J, K, XJK, stats);
	
	L1 = log_uni(J, XIJ, mu1);
	L2 = log_bi(J, K, Xnull, mu2);
	
	
	ratio1 = LRT(L1, L2);
	cout << "The LRT of the observed table is:" << ratio1 << endl << endl;
	
	srand(time(0));
	irand.seed(time(0));
	int re_run=0;
	
	for(int numOfItr = 0; numOfItr < BurnIn; numOfItr++){
		re_run = 0;
		while(OneTable_uni(I, J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 100) break;
		}
	}
	
	for(int numOfItr = 0; numOfItr < itr2*Skip; numOfItr++){
		re_run = 0;
		while(OneTable_uni(I, J, K, (&Xnull), irand) != 0){
			re_run++;
			if(re_run > 10000) break;
		}
		
		
		
		for (int j = 0; j < J; j++) {
			for (int k = 0; k < K; k++) {
				XJK[j][k] = (Xnull)[0][j][k] + (Xnull)[1][j][k];
				//				cout << XJK[j][k] << " ";
			}
			//			cout << endl;
		}
		//		cout << endl;
		
		stats = suff_stat(J, K, Xnull);
		
		
		
		mu1 = newton_uni(J, XJ, ss_null);
		mu2 = newton_bi(J, K, XJK, stats);
		
		if (mu1[0] == -10000 & mu1[1] == -10000 & mu2[0] == -10000 & mu2[1] == -10000 & mu2[2] == -10000 ) {
			itr2 = itr2 - 1;
		}
		
		else {
		L1 = log_uni(J, XIJ, mu1);
		L2 = log_bi(J, K, Xnull, mu2);
		
		
		ratio = LRT(L1, L2);
		
		if((numOfItr % Skip) == 0){
			for(int i = 0; i < I; i++){
				for(int j = 0; j < J; j++){
					for (int k = 0; k < K; k++)
						outTable << Xnull[i][j][k] << " ";
				    outTable << endl;
			    }outTable << endl;
		    }outTable << endl;
		}
		
		if((numOfItr % Skip) == 0){
			outLRT << ratio << endl;
			
			if(ratio1 <= ratio) freq++;
		}outLRT ; 	
		}
	}
	
	cout << "p-value is: " << ((double) freq)/ ((double)itr2) << endl << endl;
	t2 = clock();
	cout << "Running time is " << (float(t2) - float(t1))/CLOCKS_PER_SEC;
	return 0;
	
}

